from __future__ import print_function
from numpy import int16
import time

def connect(route,**args):
	'''
	route can either be I.I2C , or a radioLink instance
	'''
	return SHT21(route,**args)

class SHT21():	
	RESET = 0xFE
	TEMP_ADDRESS = 0xF3
	HUMIDITY_ADDRESS = 0xF5	
	selected=0xF3
	NUMPLOTS=1
	PLOTNAMES = ['Data']
	ADDRESS = 0x40
	name = 'Humidity/Temperature'
	def __init__(self,I2C,**args):
		self.I2C=I2C
		self.ADDRESS = args.get('address',self.ADDRESS)
		self.name = 'Humidity/Temperature'
		'''
		try:
			print ('switching baud to 400k')
			self.I2C.configI2C(400e3)
		except:
			print ('FAILED TO CHANGE BAUD RATE')
		'''
		self.params={'selectParameter':['temperature','humidity']}
		self.init('')


	def init(self,x):
		self.I2C.writeBulk(self.ADDRESS,[self.RESET]) #soft reset
		time.sleep(0.1)
	
	def rawToTemp(self,vals):
		if vals:
			if len(vals):
				v = (vals[0]<<8)|(vals[1]&0xFC)  #make integer & remove status bits
				v*=175.72;	v/= (1<<16);	v-=46.85
				return [v]
		return False

	def rawToRH(self,vals):
		if vals:
			if len(vals):
				v = (vals[0]<<8)|(vals[1]&0xFC)  #make integer & remove status bits
				v*=125.;	v/= (1<<16);	v-=6
				return [v]
		return False

	@staticmethod
	def _calculate_checksum(data, number_of_bytes):
		"""5.7 CRC Checksum using the polynomial given in the datasheet
		Credits: https://github.com/jaques/sht21_python/blob/master/sht21.py
		"""
		# CRC
		POLYNOMIAL = 0x131 # //P(x)=x^8+x^5+x^4+1 = 100110001
		crc = 0
		# calculates 8-Bit checksum with given polynomial
		for byteCtr in range(number_of_bytes):
			crc ^= (data[byteCtr])
			for bit in range(8, 0, -1):
				if crc & 0x80:
					crc = (crc << 1) ^ POLYNOMIAL
				else:
					crc = (crc << 1)
		return crc


	def selectParameter(self,param):
		if param=='temperature':self.selected=self.TEMP_ADDRESS
		elif param=='humidity':self.selected=self.HUMIDITY_ADDRESS

	def getRaw(self):
		self.I2C.writeBulk(self.ADDRESS,[self.selected])
		if self.selected==self.TEMP_ADDRESS:time.sleep(0.1)
		elif self.selected==self.HUMIDITY_ADDRESS:time.sleep(0.05)

		vals = self.I2C.simpleRead(self.ADDRESS,3)
		if vals:
			if self._calculate_checksum(vals,2)!=vals[2]:
				return False
				print (vals)
		if self.selected==self.TEMP_ADDRESS:return self.rawToTemp(vals)		
		elif self.selected==self.HUMIDITY_ADDRESS:return self.rawToRH(vals)		
		



# -*- coding: utf-8; mode: python; indent-tabs-mode: t; tab-width:4 -*-
import sys, time, utils, math, os.path

from QtVersion import *

import pyqtgraph as pg
import numpy as np
import eyes17.eyemath17 as em


class Expt(QWidget):
	TIMER = 50
	RPWIDTH = 300
	RPGAP = 4
	AWGmin = 1
	AWGmax = 5000
	AWGval = 1000

	tbvals = [0.050, .200, 0.500, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0]	# allowed mS/div values
	NP = 500			# Number of samples
	TG = 2				# Number of channels
	MINDEL = 1			# minimum time between samples, in usecs
	MAXDEL = 1000
	delay = MINDEL		# Time interval between samples
	TBval = 2			# timebase list index
	Res = 1000
	
	VMIN = 0
	VMAX = 5
	MAXCHAN = 1
	timeData    = [None]*MAXCHAN
	voltData    = [None]*MAXCHAN
	voltDataFit = [None]*MAXCHAN
	traceWidget = [None]*MAXCHAN
	traceWidget = [None]*MAXCHAN
	history = []		# Data store	
	traces = []
	trial = 0

	sources = ['A1','A2','A3', 'MIC']
	chanpens = ['y','g','w','m']     #pqtgraph pen colors

	
	def __init__(self, device=None):
		QWidget.__init__(self)
		self.p = device										# connection to the device hardware 
		try:
			self.p.select_range('A1',8.0)
		except:
			pass		
		self.traceCols = utils.makeTraceColors()
			
		self.history = []
		self.traces = []
		self.pwin = pg.PlotWidget()							# pyqtgraph window
		self.pwin.showGrid(x=True, y=True)					# with grid
		ax = self.pwin.getAxis('bottom')
		ax.setLabel(self.tr('Time (mS)'))	
		ax = self.pwin.getAxis('left')
		ax.setLabel(self.tr('Voltage'))
		self.pwin.disableAutoRange()
		self.pwin.setYRange(self.VMIN, self.VMAX)
		#self.pwin.hideButtons()								# Do not show the 'A' button of pg

		right = QVBoxLayout()							# right side vertical layout
		right.setAlignment(Qt.AlignTop)
		right.setSpacing(self.RPGAP)
					
		H = QHBoxLayout()
		l = QLabel(text=self.tr('Timebase'))
		l.setMaximumWidth(60)
		H.addWidget(l)
		self.TBslider = utils.slider(0, len(self.tbvals)-1, self.TBval, 150, self.set_timebase)
		H.addWidget(self.TBslider)
		l = QLabel(text=self.tr('mS/div'))
		l.setMaximumWidth(60)
		H.addWidget(l)
		right.addLayout(H)
		
		b = QPushButton(self.tr("0 -> 5V step on OD1"))
		b.clicked.connect(self.charge)		
		right.addWidget(b)
		
		b = QPushButton(self.tr("5 -> 0V step on OD1"))
		b.clicked.connect(self.discharge)		
		right.addWidget(b)

		b = QPushButton(self.tr("Calculate RC"))
		b.clicked.connect(self.fit_curve)		
		right.addWidget(b)

		b = QPushButton(self.tr("Clear Data & Traces"))
		b.clicked.connect(self.clear)		
		right.addWidget(b)
		
		H = QHBoxLayout()
		l = QLabel(self.tr("Resistance ="))
		l.setMaximumWidth(100)
		H.addWidget(l)
		self.Rval = utils.lineEdit(50, self.Res, 10, None)
		H.addWidget(self.Rval)
		l = QLabel(self.tr("Ohm"))
		l.setMaximumWidth(30)
		H.addWidget(l)
		right.addLayout(H)

		self.SaveButton = QPushButton(self.tr("Save Data"))
		self.SaveButton.clicked.connect(self.save_data)		
		right.addWidget(self.SaveButton)

		
		#------------------------end of right panel ----------------
		
		top = QHBoxLayout()
		top.addWidget(self.pwin)
		top.addLayout(right)
		
		full = QVBoxLayout()
		full.addLayout(top)
		self.msgwin = QLabel(text=self.tr('messages'))
		full.addWidget(self.msgwin)
				
		self.setLayout(full)
		
		self.timer = QTimer()
		self.timer.timeout.connect(self.update)
		self.timer.start(self.TIMER)
		self.set_timebase(self.TBval)

		#----------------------------- end of init ---------------
		
	def fit_curve(self):
		if self.history != []:
			fa = em.fit_exp(self.history[-1][0], self.history[-1][1])
		else:
			self.msg(self.tr('No data to analyze.'))
			return
		try:
			rval = float(self.Rval.text())/1000  # convert to kOhms
		except:
			return

		if fa != None:
			pa = fa[1]
			rc = abs(1.0 / pa[1])
			self.traces.append(self.pwin.plot(self.history[-1][0], fa[0], pen = self.traceCols[self.trial%5]))
			self.trial += 1
			ss = '%5.1f'%rc
			self.msg(self.tr('Fitted data with V=Vo*exp(-t/RC). RC = ') + ss + self.tr(' mSec'))
		else:
			self.msg(self.tr('Failed to fit the curve with V=Vo*exp(-t/RC)'))
	
	def charge(self):
		try:
			self.p.set_state(OD1=0)		# OD1 to LOW
			time.sleep(self.tbvals[self.TBval]*0.01)
			t,v = self.p.capture_action('A1',self.NP, self.TG,'SET_HIGH')
		except:
			self.comerr()
			return 

		self.traces.append(self.pwin.plot(t,v, pen = self.traceCols[self.trial%5]))
		self.history.append((t,v))
		self.trial += 1

	def discharge(self):
		try:
			self.p.set_state(OD1=1)		# OD1 to LOW
			time.sleep(self.tbvals[self.TBval]*0.01)
			t,v = self.p.capture_action('A1',self.NP, self.TG,'SET_LOW')
		except:
			self.comerr()
			return 

		self.traces.append(self.pwin.plot(t,v, pen = self.traceCols[self.trial%5]))
		self.history.append((t,v))
		self.trial += 1

	def clear(self):
		for k in self.traces:
			self.pwin.removeItem(k)
		self.history = []
		self.trial = 0
		self.msg(self.tr('Cleared Data and Traces'))

	def save_data(self):
		if self.history == []:
			self.msg(self.tr('No data to save'))
			return
		fn = QFileDialog.getSaveFileName()
		if fn != '':
			self.p.save(self.history, fn)
			self.msg(self.tr('Traces saved to ') + unicode(fn))				
			
	def set_timebase(self, tb):
		self.TBval = tb
		self.pwin.setXRange(0, self.tbvals[self.TBval]*10)
		msperdiv = self.tbvals[int(tb)]				#millisecs / division
		totalusec = msperdiv * 1000 * 10.0  	# total 10 divisions
		self.TG = int(totalusec/self.NP)
		if self.TG < self.MINDEL:
			self.TG = self.MINDEL
		elif self.TG > self.MAXDEL:
			self.TG = self.MAXDEL

	def set_wave(self):
		try:
			res = self.p.set_sine(self.AWGval)
			ss = '%6.2f'%res
			self.msg(self.tr('AWG set to ') + ss + self.tr(' Hz'))
		except:
			self.comerr()
			return 

	def awg_text(self, text):
		try:
			val = float(text)
		except:
			return	
		if self.AWGmin <= val <= self.AWGmax:
			self.AWGval = val
			self.AWGslider.setValue(self.AWGval)
			self.set_wave()

	def awg_slider(self, val):
		if self.AWGmin <= val <= self.AWGmax:
			self.AWGval = val
			self.AWGtext.setText(unicode(val))
			self.set_wave()
		
	def msg(self, m):
		self.msgwin.setText(self.tr(m))
		
	def comerr(self):
		self.msgwin.setText('<font color="red">' + self.tr('Error. Try Device->Reconnect'))

if __name__ == '__main__':
	import eyes17.eyes
	dev = eyes17.eyes.open()
	app = QApplication(sys.argv)

	# translation stuff
	lang=QLocale.system().name()
	t=QTranslator()
	t.load("lang/"+lang, os.path.dirname(__file__))
	app.installTranslator(t)
	t1=QTranslator()
	t1.load("qt_"+lang,
	        QLibraryInfo.location(QLibraryInfo.TranslationsPath))
	app.installTranslator(t1)

	mw = Expt(dev)
	mw.show()
	sys.exit(app.exec_())
	



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'miniScope.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(518, 354)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Form)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(1)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.splitter = QtWidgets.QSplitter(Form)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.frame_2 = QtWidgets.QFrame(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(90)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
        self.frame_2.setSizePolicy(sizePolicy)
        self.frame_2.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.frame_2)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.plot = PlotWidget(self.frame_2)
        self.plot.setObjectName("plot")
        self.verticalLayout.addWidget(self.plot)
        self.message = QtWidgets.QLabel(self.frame_2)
        self.message.setStyleSheet("color: rgb(239, 41, 41);\n"
"font: 57 12pt \"Ubuntu\";")
        self.message.setText("")
        self.message.setObjectName("message")
        self.verticalLayout.addWidget(self.message)
        self.frame = QtWidgets.QFrame(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setVerticalSpacing(1)
        self.gridLayout.setObjectName("gridLayout")
        self.A1Enabled = QtWidgets.QCheckBox(self.frame)
        self.A1Enabled.setText("")
        self.A1Enabled.setChecked(True)
        self.A1Enabled.setObjectName("A1Enabled")
        self.gridLayout.addWidget(self.A1Enabled, 2, 0, 1, 1)
        self.A1Box = QtWidgets.QComboBox(self.frame)
        self.A1Box.setObjectName("A1Box")
        self.gridLayout.addWidget(self.A1Box, 2, 1, 1, 2)
        self.label_2 = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 3)
        self.frame_3 = QtWidgets.QFrame(self.frame)
        self.frame_3.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_3.setObjectName("frame_3")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.frame_3)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setSpacing(2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.timebaseSlider = QtWidgets.QSlider(self.frame_3)
        self.timebaseSlider.setMaximum(7)
        self.timebaseSlider.setOrientation(QtCore.Qt.Horizontal)
        self.timebaseSlider.setTickPosition(QtWidgets.QSlider.TicksAbove)
        self.timebaseSlider.setObjectName("timebaseSlider")
        self.horizontalLayout_2.addWidget(self.timebaseSlider)
        self.timebaseLabel = QtWidgets.QLabel(self.frame_3)
        self.timebaseLabel.setObjectName("timebaseLabel")
        self.horizontalLayout_2.addWidget(self.timebaseLabel)
        self.gridLayout.addWidget(self.frame_3, 0, 0, 1, 3)
        self.gridLayout.setColumnStretch(2, 10)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.A2Box = QtWidgets.QCheckBox(self.frame)
        self.A2Box.setObjectName("A2Box")
        self.verticalLayout_2.addWidget(self.A2Box)
        self.label = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.verticalLayout_2.addWidget(self.label)
        self.line = QtWidgets.QFrame(self.frame)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.resultsLayout = QtWidgets.QVBoxLayout()
        self.resultsLayout.setObjectName("resultsLayout")
        self.list = QtWidgets.QListWidget(self.frame)
        self.list.setObjectName("list")
        self.resultsLayout.addWidget(self.list)
        self.verticalLayout_2.addLayout(self.resultsLayout)
        self.freqCheckBox = QtWidgets.QCheckBox(self.frame)
        self.freqCheckBox.setObjectName("freqCheckBox")
        self.verticalLayout_2.addWidget(self.freqCheckBox)
        self.verticalLayout_2.setStretch(4, 10)
        self.horizontalLayout.addWidget(self.splitter)

        self.retranslateUi(Form)
        self.list.currentRowChanged['int'].connect(Form.changeParameter)
        self.A1Box.currentTextChanged['QString'].connect(Form.changeChannel)
        self.timebaseSlider.valueChanged['int'].connect(Form.timebaseChanged)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label_2.setText(_translate("Form", "Channels"))
        self.timebaseLabel.setText(_translate("Form", "1mS"))
        self.A2Box.setText(_translate("Form", "A2"))
        self.label.setText(_translate("Form", "Results"))
        self.freqCheckBox.setText(_translate("Form", "Cross Check Freq"))

from pyqtgraph import PlotWidget



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dio_control.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

import sys
if sys.version_info.major==3:
	from PyQt5 import QtGui, QtCore, QtWidgets
else:
	from PyQt4 import QtGui, QtCore
	from PyQt4 import QtGui as QtWidgets
	
class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(363, 300)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setContentsMargins(3, 3, 3, 0)
        self.gridLayout.setSpacing(3)
        self.gridLayout.setObjectName("gridLayout")
        self.configLayout = QtWidgets.QHBoxLayout()
        self.configLayout.setObjectName("configLayout")
        self.gridLayout.addLayout(self.configLayout, 1, 0, 1, 3)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 1, 1, 1)
        self.pushButton = QtWidgets.QPushButton(Dialog)
        self.pushButton.setObjectName("pushButton")
        self.gridLayout.addWidget(self.pushButton, 0, 0, 1, 1)
        self.monitors = QtWidgets.QStackedWidget(Dialog)
        self.monitors.setObjectName("monitors")
        self.page = QtWidgets.QWidget()
        self.page.setObjectName("page")
        self.gaugeLayout = QtWidgets.QHBoxLayout(self.page)
        self.gaugeLayout.setContentsMargins(5, 5, 5, 5)
        self.gaugeLayout.setObjectName("gaugeLayout")
        self.monitors.addWidget(self.page)
        self.page_2 = QtWidgets.QWidget()
        self.page_2.setObjectName("page_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.page_2)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.graph = PlotWidget(self.page_2)
        self.graph.setObjectName("graph")
        self.horizontalLayout_2.addWidget(self.graph)
        self.monitors.addWidget(self.page_2)
        self.gridLayout.addWidget(self.monitors, 2, 0, 1, 4)

        self.retranslateUi(Dialog)
        self.monitors.setCurrentIndex(0)
        self.pushButton.clicked.connect(Dialog.initialize)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Control"))
        self.pushButton.setText(_translate("Dialog", "Initialize"))

from pyqtgraph import PlotWidget
from . import res_rc



'''
Code for studying newton's laws using sr04 sensor
Logs data from various sensors.
Author  : Jithin B.P, jithinbp@gmail.com
Date    : Sep-2019
License : GNU GPL version 3
'''
import sys,time
t = time.time()

from utils import *
if sys.version_info.major==3:
	from PyQt5 import QtGui, QtCore, QtWidgets
else:
	from PyQt4 import QtGui, QtCore
	from PyQt4 import QtGui as QtWidgets

import pyqtgraph as pg

import math, os.path, struct
from collections import OrderedDict

from layouts import ui_newtonslaws

from layouts.gauge import Gauge
import functools
from functools import partial


import numpy as np
import math
import numpy.linalg

print(time.time()-t)
colors=['#00ff00','#ff0000','#ffff80',(10,255,255)]+[(50+np.random.randint(200),50+np.random.randint(200),150+np.random.randint(100)) for a in range(10)]

Byte =     struct.Struct("B") # size 1
ShortInt = struct.Struct("H") # size 2
Integer=   struct.Struct("I") # size 4

############# MATHEMATICAL AND ANALYTICS ###############

#TODO

############# MATHEMATICAL AND ANALYTICS ###############


class Expt(QtWidgets.QMainWindow, ui_newtonslaws.Ui_MainWindow):
	def __init__(self, device=None):
		super(Expt, self).__init__()
		self.setupUi(self)

		self.p = device

		self.rec = True #Recording enabled

		#Define some keyboard shortcuts for ease of use
		self.shortcutActions={}
		self.shortcuts={" ":self.toggleRecord}
		for a in self.shortcuts:
			shortcut = QtWidgets.QShortcut(QtGui.QKeySequence(a), self)
			shortcut.activated.connect(self.shortcuts[a])
			self.shortcutActions[a] = shortcut

		self.T = 0
		self.start_time = time.time()
		self.points=0
		self.time = np.empty(300)
		self.pcurve_data = np.empty(300)
		self.vcurve_data = []
		self.acurve_data = []

		self.pcurve = self.graphPosition.plot(pen=colors[0])
		self.graphPosition.setRange(xRange=[-5, 0])

		self.pcurve2 = self.graphResults.plot(pen=colors[0],name='Position')
		self.graphResults.setLabel('left', "Position", units='cm',color='#00ff00')

		self.graphResults.viewBoxes=[]

		## Add secondary Axis for Velocity
		self.p3 = pg.ViewBox()
		ax3 = pg.AxisItem('right')
		self.graphResults.plotItem.layout.addItem(ax3, 2, 3) 
		self.graphResults.plotItem.scene().addItem(self.p3)
		ax3.linkToView(self.p3)
		self.p3.setXLink(self.graphResults)
		ax3.setZValue(-10000)
		ax3.setLabel('Velocity', color=colors[1])
		self.p3.setGeometry(self.graphResults.plotItem.vb.sceneBoundingRect())
		self.p3.linkedViewChanged(self.graphResults.plotItem.vb, self.p3.XAxis)
		self.graphResults.viewBoxes.append(self.p3)
		self.vcurve = pg.PlotDataItem(pen=colors[1],name='Velocity')
		self.p3.addItem(self.vcurve)

		## Add third Axis for Acceleration
		self.p2 = pg.ViewBox()
		ax2 = pg.AxisItem('right')
		self.graphResults.plotItem.layout.addItem(ax2, 2, 4) 
		self.graphResults.plotItem.scene().addItem(self.p2)
		ax2.linkToView(self.p2)
		self.p2.setXLink(self.graphResults)
		ax2.setZValue(-10000)
		ax2.setLabel('Acceleration', color=colors[2])
		self.p2.setGeometry(self.graphResults.plotItem.vb.sceneBoundingRect())
		self.p2.linkedViewChanged(self.graphResults.plotItem.vb, self.p2.XAxis)
		self.graphResults.viewBoxes.append(self.p2)
		self.acurve = pg.PlotDataItem(pen=colors[2],name='Acceleration')
		self.p2.addItem(self.acurve)

		Callback = functools.partial(self.updateViews,self.graphResults)
		self.graphResults.getViewBox().sigStateChanged.connect(Callback)

		self.graphResults.setRange(xRange=[0, 5])

		self.region = pg.LinearRegionItem()
		self.region.setBrush([255,0,50,50])
		self.region.setZValue(10)
		for a in self.region.lines: a.setCursor(QtGui.QCursor(QtCore.Qt.SizeHorCursor)); 
		self.graphPosition.addItem(self.region, ignoreBounds=False)
		self.region.setRegion([-3,-.5])

		#cross hair
		self.vLine = pg.InfiniteLine(angle=90, movable=False)
		self.hLine = pg.InfiniteLine(angle=0, movable=False)
		self.graphResults.addItem(self.vLine, ignoreBounds=True)
		self.graphResults.addItem(self.hLine, ignoreBounds=True)
		self.graphResults.setTitle('Pause acquisition to view derivates here',justify='left')
		self.resultslabel = self.graphResults.plotItem.titleLabel
		self.proxy = pg.SignalProxy(self.graphResults.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved)

		# Populate contents of samplesBox(For moving average) and polyBox(Degree of polynomial)
		self.samplesBox.addItems(['30']+[str(a*5) for a in range(1,20)])
		self.polyBox.addItems([str(a) for a in range(2,7)])
		self.samplesBox.setCurrentIndex(5)
		self.samplesBox.setCurrentIndex(0)

		self.gauge = Gauge(self)
		self.gauge.setObjectName("distance")
		self.gauge.set_MinValue(0)
		self.gauge.set_MaxValue(50)
		self.gaugeLayout.addWidget(self.gauge)

		self.startTime = time.time()
		self.timer = QtCore.QTimer()
		self.timer.timeout.connect(self.updateEverything)
		self.timer.start(2)

	def mouseMoved(self,evt):
		pos = evt[0]  ## using signal proxy turns original arguments into a tuple
		vb = self.graphResults.plotItem.vb
		if vb.sceneBoundingRect().contains(pos):
			mousePoint = vb.mapSceneToView(pos)
			index = int(np.abs(self.time - mousePoint.x()).argmin()) #(np.abs(arr-val)).argmin()
			if index > 0 and index < len(self.vcurve_data):
				self.resultslabel.setText("<span style='font-size: 12pt'>x=%0.3f,   <span style='color: green'>P=%0.1f cm</span>,<span style='color: red'>V=%0.2f</span>,   <span style='color: yellow'>A=%0.2f</span>" % (mousePoint.x(), self.pcurve_data[index], self.vcurve_data[index], self.acurve_data[index]))
			self.vLine.setPos(mousePoint.x())
			self.hLine.setPos(mousePoint.y())


	def updateViews(self,plot):
			for a in plot.viewBoxes:
				a.setGeometry(plot.getViewBox().sceneBoundingRect())
				a.linkedViewChanged(plot.plotItem.vb, a.XAxis)

	def updateEverything(self):
		if self.rec:
			D = self.p.sr04_distance()
			if D is None: return
			self.gauge.update_value(D)
			self.T = time.time() - self.start_time
			self.time[self.points] = self.T
			self.pcurve_data[self.points] = D
			self.pcurve.setData(self.time[:self.points],self.pcurve_data[:self.points])
			self.pcurve.setPos(-self.T, 0)


			self.points+=1
			if self.points >= self.time.shape[0]-1:
				tmp = self.time
				self.time = np.empty(self.time.shape[0] * 2) #double the size
				self.time[:tmp.shape[0]] = tmp
				tmp = self.pcurve_data
				self.pcurve_data = np.empty(self.pcurve_data.shape[0] * 2) #double the size
				self.pcurve_data[:tmp.shape[0]] = tmp


	### ANALYSIS : https://dsp.stackexchange.com/a/9512
	### Accepted answer on using moving averages for this particular problem.

	def sg_filter(self,x, m, k=0):
		"""
		x = Vector of sample times
		m = Order of the smoothing polynomial
		k = Which derivative
		"""
		mid = int(len(x) / 2)        
		a = x - x[mid]
		expa = lambda x: list(map(lambda i: i**x, a))
		mp = list(map(expa, range(0,m+1)))
		A = np.r_[mp].transpose()
		Ai = np.linalg.pinv(A)
		return Ai[k]

	def smooth(self,x, y, size=5, order=2, deriv=0):
		if deriv > order:
			print( "deriv must be <= order")
			return None

		n = len(x)
		m = size
		result = np.zeros(n)
		for i in range(m, n-m):
			start, end = i - m, i + m + 1
			f = self.sg_filter(np.array(x[start:end]), order, deriv)
			result[i] = np.dot(f, np.array(y[start:end]))

		if deriv > 1:
			result *= math.factorial(deriv)

		return result

	def redoAnalysis(self,s):
		if not self.rec: #Analysis mode
			self.analysis()

	def analysis(self):
		self.recordBox.setChecked(False)
		self.setRecord(False)

		T = np.array(self.time[:self.points])
		pos = np.array(self.pcurve_data[:self.points])

		if self.splineBox.currentIndex()!=0:
			z = np.polyfit(T,pos,10)
			zp = np.poly1d(z)
			pos = zp(T)


		self.pcurve2.setData(T,pos)
		smoothing_size = int(self.samplesBox.currentText()) # points for smoothing
		order = int(self.polyBox.currentText()) #  degree of polynomial

		params = (T, pos, smoothing_size, order)
		V = self.smooth(*params, deriv=1) #1st derivative
		A = self.smooth(*params, deriv=2) #2nd derivative

		self.vcurve_data = V
		self.acurve_data = A

		#self.pcurve2.setData(T,pos)
		self.vcurve.setData(T,V)
		self.acurve.setData(T,A)
		self.graphResults.autoRange()
		self.p2.autoRange()
		self.p3.autoRange()

	def clearAll(self):
		self.T = 0
		self.start_time = time.time()
		self.points=0
		self.time = np.empty(300)
		self.pcurve_data = np.empty(300)
		self.vcurve_data = []
		self.acurve_data = []

		self.graphPosition.setRange(xRange=[-5, 0])
		self.pcurve.setPos(0, 0)

		for a in [self.pcurve,self.pcurve2,self.vcurve,self.acurve]:
			a.clear()

	def setRecord(self,val):
		self.rec = val
		if self.rec:
			self.clearAll()
			self.resultslabel.setText('Acquiring data...')
			self.splitter.setSizes([200,100])
		else:
			self.splitter.setSizes([100,200])
	def toggleRecord(self):
		if self.rec:
			self.recordBox.setChecked(False)
			self.setRecord(False)
			self.analysis()
		else:
			self.recordBox.setChecked(True)
			self.setRecord(True)

if __name__ == '__main__':
	import eyes17.eyes
	dev = eyes17.eyes.open()
	app = QtWidgets.QApplication(sys.argv)

	# translation stuff
	lang=QtCore.QLocale.system().name()
	t=QtCore.QTranslator()
	t.load("lang/"+lang, os.path.dirname(__file__))
	app.installTranslator(t)
	t1=QtCore.QTranslator()
	t1.load("qt_"+lang,
	        QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath))
	app.installTranslator(t1)
	mw = Expt(dev)
	mw.show()
	sys.exit(app.exec_())
	



'''
Code for studying newton's laws using sr04 sensor
Logs data from various sensors.
Author  : Jithin B.P, jithinbp@gmail.com
Date    : Sep-2019
License : GNU GPL version 3
'''
import sys

if sys.version_info.major==3:
	from PyQt5 import QtGui, QtCore, QtWidgets
else:
	from PyQt4 import QtGui, QtCore
	from PyQt4 import QtGui as QtWidgets

import time, utils, math, os.path, struct
from collections import OrderedDict

from layouts import ui_scope

from layouts.gauge import Gauge
import functools
from functools import partial
import time



from pyqtgraph.exporters import Exporter
from pyqtgraph.parametertree import Parameter
from pyqtgraph.Qt import QtGui, QtCore, QtSvg, USE_PYSIDE
from pyqtgraph import functions as fn
import pyqtgraph as pg

__all__ = ['PQG_ImageExporter']


class PQG_ImageExporter(Exporter):
    Name = "Working Image Exporter (PNG, TIF, JPG, ...)"
    allowCopy = True

    def __init__(self, item):
        Exporter.__init__(self, item)
        tr = self.getTargetRect()
        if isinstance(item, QtGui.QGraphicsItem):
            scene = item.scene()
        else:
            scene = item
        # scene.views()[0].backgroundBrush()
        bgbrush = pg.mkBrush('w')
        bg = bgbrush.color()
        if bgbrush.style() == QtCore.Qt.NoBrush:
            bg.setAlpha(0)

        self.params = Parameter(name='params', type='group', children=[
            {'name': 'width', 'type': 'int',
                'value': tr.width(), 'limits': (0, None)},
            {'name': 'height', 'type': 'int',
                'value': tr.height(), 'limits': (0, None)},
            {'name': 'antialias', 'type': 'bool', 'value': True},
            {'name': 'background', 'type': 'color', 'value': bg},
        ])
        self.params.param('width').sigValueChanged.connect(self.widthChanged)
        self.params.param('height').sigValueChanged.connect(self.heightChanged)

    def widthChanged(self):
        sr = self.getSourceRect()
        ar = float(sr.height()) / sr.width()
        self.params.param('height').setValue(
            self.params['width'] * ar, blockSignal=self.heightChanged)

    def heightChanged(self):
        sr = self.getSourceRect()
        ar = float(sr.width()) / sr.height()
        self.params.param('width').setValue(
            self.params['height'] * ar, blockSignal=self.widthChanged)

    def parameters(self):
        return self.params

    def export(self, fileName=None, toBytes=False, copy=False):
        if fileName is None and not toBytes and not copy:
            if USE_PYSIDE:
                filter = ["*."+str(f)
                          for f in QtGui.QImageWriter.supportedImageFormats()]
            else:
                filter = ["*."+bytes(f).decode('utf-8')
                          for f in QtGui.QImageWriter.supportedImageFormats()]
            preferred = ['*.png', '*.tif', '*.jpg']
            for p in preferred[::-1]:
                if p in filter:
                    filter.remove(p)
                    filter.insert(0, p)
            self.fileSaveDialog(filter=filter)
            return

        targetRect = QtCore.QRect(
            0, 0, self.params['width'], self.params['height'])
        sourceRect = self.getSourceRect()

        #self.png = QtGui.QImage(targetRect.size(), QtGui.QImage.Format_ARGB32)
        # self.png.fill(pyqtgraph.mkColor(self.params['background']))
        w, h = self.params['width'], self.params['height']
        if w == 0 or h == 0:
            raise Exception(
                "Cannot export image with size=0 (requested export size is %dx%d)" % (w, h))
        bg = np.empty((int(self.params['width']), int(
            self.params['height']), 4), dtype=np.ubyte)
        color = self.params['background']
        bg[:, :, 0] = color.blue()
        bg[:, :, 1] = color.green()
        bg[:, :, 2] = color.red()
        bg[:, :, 3] = color.alpha()
        self.png = fn.makeQImage(bg, alpha=True)

        # set resolution of image:
        origTargetRect = self.getTargetRect()
        resolutionScale = targetRect.width() / origTargetRect.width()
        #self.png.setDotsPerMeterX(self.png.dotsPerMeterX() * resolutionScale)
        #self.png.setDotsPerMeterY(self.png.dotsPerMeterY() * resolutionScale)

        painter = QtGui.QPainter(self.png)
        #dtr = painter.deviceTransform()
        try:
            self.setExportMode(True, {
                               'antialias': self.params['antialias'], 'background': self.params['background'], 'painter': painter, 'resolutionScale': resolutionScale})
            painter.setRenderHint(
                QtGui.QPainter.Antialiasing, self.params['antialias'])
            self.getScene().render(painter, QtCore.QRectF(
                targetRect), QtCore.QRectF(sourceRect))
        finally:
            self.setExportMode(False)
        painter.end()

        if copy:
            QtGui.QApplication.clipboard().setImage(self.png)
        elif toBytes:
            return self.png
        else:
            self.png.save(fileName)


PQG_ImageExporter.register()

import numpy as np
import math
import numpy.linalg

colors=['#00ff00','#ff0000','#ffff80',(10,255,255)]+[(50+np.random.randint(200),50+np.random.randint(200),150+np.random.randint(100)) for a in range(10)]

Byte =     struct.Struct("B") # size 1
ShortInt = struct.Struct("H") # size 2
Integer=   struct.Struct("I") # size 4

############# MATHEMATICAL AND ANALYTICS ###############

#TODO

############# MATHEMATICAL AND ANALYTICS ###############

class task:
	def __init__(self,func):
		self.func = func
		self.args=None
		self.kwargs=None
		self.update = False
		
	def set(self,*args,**kwargs):
		self.args = args
		self.kwargs = kwargs
		self.update = True
	def run(self):
		self.func(*self.args,**self.kwargs)
		self.update = False

class communicationHandler(QtCore.QObject):
	sigStat = QtCore.pyqtSignal(str,bool)

	def __init__(self, **kwargs):
		super(self.__class__, self).__init__()
		self.p = kwargs.get('dev',None)

		self.tasks = OrderedDict()
		for a in ['set_sine']:
			self.tasks[a] = task(getattr(self.p,a))

	def update(self,func,args=[],kwargs={}):
		self.tasks[func].set(*args,**kwargs)

	def run(self):
		while 1:
			print(time.time(),self.tasks['set_sine'].update)

class Expt(QtWidgets.QMainWindow, ui_scope.Ui_MainWindow):
	sigExec = QtCore.pyqtSignal(str,list,dict)
	def __init__(self, parent=None,device=None):
		super(Expt, self).__init__(parent)
		self.setupUi(self)


		### Prepare the communication handler, and move it to a thread.
		self.CH = communicationHandler(dev=device)
		self.worker_thread = QtCore.QThread()
		self.CH.moveToThread(self.worker_thread)
		self.sigExec.connect(self.CH.update)

		self.worker_thread.start()
		self.worker_thread.setPriority(QtCore.QThread.HighPriority)



		#Define some keyboard shortcuts for ease of use
		self.shortcutActions={}
		self.shortcuts={" ":self.toggleRecord}
		for a in self.shortcuts:
			shortcut = QtWidgets.QShortcut(QtGui.QKeySequence(a), self)
			shortcut.activated.connect(self.shortcuts[a])
			self.shortcutActions[a] = shortcut


		self.T = 0
		self.start_time = time.time()
		self.points=0
		self.time = np.empty(300)
		self.pcurve_data = np.empty(300)
		self.vcurve_data = []
		self.acurve_data = []

		self.pcurve = self.graphPosition.plot(pen=colors[0])
		self.graphPosition.setRange(xRange=[0, 2],yRange=[-5, 5])

		#cross hair
		self.vLine = pg.InfiniteLine(angle=90, movable=False)
		self.hLine = pg.InfiniteLine(angle=0, movable=False)
		self.graphPosition.addItem(self.vLine, ignoreBounds=True)
		self.graphPosition.addItem(self.hLine, ignoreBounds=True)
		self.graphPosition.setTitle('Pause acquisition to view derivates here',justify='left')
		self.resultslabel = self.graphPosition.plotItem.titleLabel
		self.proxy = pg.SignalProxy(self.graphPosition.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved)

		self.splitter_2.setSizes([500,100])

		self.gauges=OrderedDict()
		row = 1; col=1;
		self.analogs = {'A1':[-16,16],'A2':[-16,16],'A3':[-3,3],'SEN':[0,3.3],'AN8':[0,3.3]}
		for a in self.analogs:
			gauge = Gauge(self,a)
			gauge.setObjectName(a)
			gauge.set_MinValue(self.analogs[a][0])
			gauge.set_MaxValue(self.analogs[a][1])
			self.gaugeLayout.addWidget(gauge,row,col)
			col+= 1
			if col == 3:
				row += 1
				col = 1
			self.gauges[a] = gauge

		self.startTime = time.time()
		self.timer = QtCore.QTimer()
		self.timer.timeout.connect(self.updateEverything)
		self.timer.start(50)

	def set_sine(self,val):
		self.sigExec.emit('set_sine',[val],{})
		#self.tasks['set_sine'].set(val)

	def mouseMoved(self,evt):
		pos = evt[0]  ## using signal proxy turns original arguments into a tuple
		vb = self.graphPosition.plotItem.vb
		if vb.sceneBoundingRect().contains(pos):
			mousePoint = vb.mapSceneToView(pos)
			index = int(np.abs(self.time - mousePoint.x()).argmin()) #(np.abs(arr-val)).argmin()
			if index > 0 and index < len(self.vcurve_data):
				self.resultslabel.setText("<span style='font-size: 12pt'>x=%0.3f,   <span style='color: green'>P=%0.1f cm</span>,<span style='color: red'>V=%0.2f</span>,   <span style='color: yellow'>A=%0.2f</span>" % (mousePoint.x(), self.pcurve_data[index], self.vcurve_data[index], self.acurve_data[index]))
			self.vLine.setPos(mousePoint.x())
			self.hLine.setPos(mousePoint.y())


	def updateViews(self,plot):
			for a in plot.viewBoxes:
				a.setGeometry(plot.getViewBox().sceneBoundingRect())
				a.linkedViewChanged(plot.plotItem.vb, a.XAxis)

	def updateEverything(self):
		for a in self.analogs:
			#D = self.p.get_average_voltage(a)
			D = 1
			self.gauges[a].update_value(D)
		#for a in self.tasks:
		#	if self.tasks[a].update:
		#		self.tasks[a].run()
		#		break

		'''
		now = time.time()
		dt = now - self.lastTime
		self.lastTime = now
		if self.fps is None:
			self.fps = 1.0/dt
		else:
			s = np.clip(dt*3., 0, 1)
			self.fps = self.fps * (1-s) + (1.0/dt) * s
		'''
		#self.pcurve.setData(self.time[:self.points],self.pcurve_data[:self.points])

	def clearAll(self):
		self.T = 0
		self.start_time = time.time()
		self.points=0
		self.time = np.empty(300)
		self.pcurve_data = np.empty(300)
		self.vcurve_data = []
		self.acurve_data = []

		self.graphPosition.setRange(xRange=[-5, 0])
		self.pcurve.setPos(0, 0)

		for a in [self.pcurve,self.pcurve2,self.vcurve,self.acurve]:
			a.clear()

	def setRecord(self,val):
		self.rec = val
		if self.rec:
			self.clearAll()
			self.resultslabel.setText('Acquiring data...')
			self.splitter.setSizes([200,100])
		else:
			self.splitter.setSizes([100,200])
	def toggleRecord(self):
		if self.rec:
			self.recordBox.setChecked(False)
			self.setRecord(False)
			self.analysis()
		else:
			self.recordBox.setChecked(True)
			self.setRecord(True)

if __name__ == '__main__':
	import eyes17.eyes
	dev = eyes17.eyes.open()
	app = QtWidgets.QApplication(sys.argv)

	# translation stuff
	lang=QtCore.QLocale.system().name()
	t=QtCore.QTranslator()
	t.load("lang/"+lang, os.path.dirname(__file__))
	app.installTranslator(t)
	t1=QtCore.QTranslator()
	t1.load("qt_"+lang,
	        QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath))
	app.installTranslator(t1)
	class dummydev:
		def sr04_distance(self):
			return np.random.random(1)
	mw = Expt(None,dev)
	mw.show()
	sys.exit(app.exec_())
	



# -*- coding: utf-8 -*-
# MF522 - Software stack to access the MF522 RFID reader via the SEELablet
#
# Copyright (C) 2015 by Jithin B.P. <jithinbp@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import print_function
from SEEL import interface
import time
def connect(I,cs):
    return MF522(I,cs)

class MF522:
    #Constants from https://github.com/miguelbalboa/rfid ( Open source License : UNLICENSE)
    CommandReg                = 0x01 << 1        # starts and stops command execution
    ComIEnReg                = 0x02 << 1        # enable and disable interrupt request control bits
    DivIEnReg                = 0x03 << 1        # enable and disable interrupt request control bits
    ComIrqReg                = 0x04 << 1        # interrupt request bits
    DivIrqReg                = 0x05 << 1        # interrupt request bits
    ErrorReg                = 0x06 << 1        # error bits showing the error status of the last command executed
    Status1Reg                = 0x07 << 1        # communication status bits
    Status2Reg                = 0x08 << 1        # receiver and transmitter status bits
    FIFODataReg                = 0x09 << 1        # input and output of 64 byte FIFO buffer
    FIFOLevelReg                = 0x0A << 1        # number of bytes stored in the FIFO buffer
    WaterLevelReg                = 0x0B << 1        # level for FIFO underflow and overflow warning
    ControlReg                = 0x0C << 1        # miscellaneous control registers
    BitFramingReg                = 0x0D << 1        # adjustments for bit-oriented frames
    CollReg                    = 0x0E << 1        # bit position of the first bit-collision detected on the RF interface

    ModeReg                    = 0x11 << 1        # defines general modes for transmitting and receiving
    TxModeReg                = 0x12 << 1        # defines transmission data rate and framing
    RxModeReg                = 0x13 << 1        # defines reception data rate and framing
    TxControlReg                = 0x14 << 1        # controls the logical behavior of the antenna driver pins TX1 and TX2
    TxASKReg                = 0x15 << 1        # controls the setting of the transmission modulation
    TxSelReg                = 0x16 << 1        # selects the internal sources for the antenna driver
    RxSelReg                = 0x17 << 1        # selects internal receiver settings
    RxThresholdReg                = 0x18 << 1        # selects thresholds for the bit decoder
    DemodReg                = 0x19 << 1        # defines demodulator settings
    MfTxReg                    = 0x1C << 1        # controls some MIFARE communication transmit parameters
    MfRxReg                    = 0x1D << 1        # controls some MIFARE communication receive parameters
    SerialSpeedReg                = 0x1F << 1        # selects the speed of the serial UART interface

    CRCResultRegH                = 0x21 << 1        # shows the MSB and LSB values of the CRC calculation
    CRCResultRegL                = 0x22 << 1
    ModWidthReg                = 0x24 << 1        # controls the ModWidth setting?
    RFCfgReg                = 0x26 << 1        # configures the receiver gain
    GsNReg                    = 0x27 << 1        # selects the conductance of the antenna driver pins TX1 and TX2 for modulation
    CWGsPReg                = 0x28 << 1        # defines the conductance of the p-driver output during periods of no modulation
    ModGsPReg                = 0x29 << 1        # defines the conductance of the p-driver output during periods of modulation
    TModeReg                = 0x2A << 1        # defines settings for the internal timer
    TPrescalerReg                = 0x2B << 1        # the lower 8 bits of the TPrescaler value. The 4 high bits are in TModeReg.
    TReloadRegH                = 0x2C << 1        # defines the 16-bit timer reload value
    TReloadRegL                = 0x2D << 1
    TCounterValueRegH            = 0x2E << 1        # shows the 16-bit timer value
    TCounterValueRegL            = 0x2F << 1

    TestSel1Reg                = 0x31 << 1        # general test signal configuration
    TestSel2Reg                = 0x32 << 1        # general test signal configuration
    TestPinEnReg                = 0x33 << 1        # enables pin output driver on pins D1 to D7
    TestPinValueReg                = 0x34 << 1        # defines the values for D1 to D7 when it is used as an I/O bus
    TestBusReg                = 0x35 << 1        # shows the status of the internal test bus
    AutoTestReg                = 0x36 << 1        # controls the digital self test
    VersionReg                = 0x37 << 1        # shows the software version
    AnalogTestReg                = 0x38 << 1        # controls the pins AUX1 and AUX2
    TestDAC1Reg                = 0x39 << 1        # defines the test value for TestDAC1
    TestDAC2Reg                = 0x3A << 1        # defines the test value for TestDAC2
    TestADCReg                = 0x3B << 1       # shows the value of ADC I and Q channels

    # MFRC522 commands. Described in chapter 10 of the datasheet.
    PCD_Idle                = 0x00            #no action, cancels current command execution
    PCD_Mem                    = 0x01            #stores 25 bytes into the internal buffer
    PCD_GenerateRandomID    = 0x02                    #generates a 10-byte random ID number
    PCD_CalcCRC                = 0x03                #activates the CRC coprocessor or performs a self test
    PCD_Transmit                = 0x04                # transmits data from the FIFO buffer
    PCD_NoCmdChange                = 0x07
    PCD_Receive                = 0x08                #activates the receiver circuits
    PCD_Transceive                 = 0x0C                #transmits data from FIFO buffer to antenna and automatically activates the receiver after transmission
    PCD_MFAuthent                 = 0x0E                #performs the MIFARE standard authentication as a reader
    PCD_SoftReset                = 0x0F                #resets the MFRC522

    RxGain_18dB                = 0x00 << 4        # 000b - 18 dB, minimum
    RxGain_23dB                = 0x01 << 4        # 001b - 23 dB
    RxGain_18dB_2                = 0x02 << 4        # 010b - 18 dB, it seems 010b is a duplicate for 000b
    RxGain_23dB_2                = 0x03 << 4        # 011b - 23 dB, it seems 011b is a duplicate for 001b
    RxGain_33dB                = 0x04 << 4        # 100b - 33 dB, average, and typical default
    RxGain_38dB                = 0x05 << 4        # 101b - 38 dB
    RxGain_43dB                = 0x06 << 4        # 110b - 43 dB
    RxGain_48dB                = 0x07 << 4        # 111b - 48 dB, maximum
    RxGain_min                = 0x00 << 4        # 000b - 18 dB, minimum, convenience for RxGain_18dB
    RxGain_avg                = 0x04 << 4        # 100b - 33 dB, average, convenience for RxGain_33dB
    RxGain_max                = 0x07 << 4        # 111b - 48 dB, maximum, convenience for RxGain_48dB

    # The commands used by the PCD to manage communication with several PICCs (ISO 14443-3, Type A, section 6.4)
    PICC_CMD_REQA            = 0x26                 # REQuest command, Type A. Invites PICCs in state IDLE to go to READY and prepare for anticollision or selection
    PICC_CMD_WUPA            = 0x52                 # Wake-UP command, prepare for anticollision or selection. 7 bit frame.
    PICC_CMD_CT            = 0x88                 # Cascade Tag. Not really a command, but used during anti collision.
    PICC_CMD_SEL_CL1        = 0x93                 # Anti collision/Select, Cascade Level 1
    PICC_CMD_SEL_CL2        = 0x95                 # Anti collision/Select, Cascade Level 2
    PICC_CMD_SEL_CL3        = 0x97                 # Anti collision/Select, Cascade Level 3
    PICC_CMD_HLTA            = 0x50                 # HaLT command, Type A. Instructs an ACTIVE PICC to go to state HALT.
    # The commands used for MIFARE Classic (from http://www.mouser.com/ds/2/302/MF1S503x-89574.pdf, Section 9)
    # Use PCD_MFAuthent to authenticate access to a sector, then use these commands to read/write/modify the blocks on the sector.
    # The read/write commands can also be used for MIFARE Ultralight.
    PICC_CMD_MF_AUTH_KEY_A    = 0x60                 # Perform authentication with Key A
    PICC_CMD_MF_AUTH_KEY_B    = 0x61                 # Perform authentication with Key B
    PICC_CMD_MF_READ        = 0x30                 # Reads one 16 byte block from the authenticated sector of the PICC. Also used for MIFARE Ultralight.
    PICC_CMD_MF_WRITE        = 0xA0                 # Writes one 16 byte block to the authenticated sector of the PICC. Called "COMPATIBILITY WRITE" for MIFARE Ultralight.
    PICC_CMD_MF_DECREMENT    = 0xC0                 # Decrements the contents of a block and stores the result in the internal data register.
    PICC_CMD_MF_INCREMENT    = 0xC1                 # Increments the contents of a block and stores the result in the internal data register.
    PICC_CMD_MF_RESTORE        = 0xC2                 # Reads the contents of a block into the internal data register.
    PICC_CMD_MF_TRANSFER    = 0xB0                 # Writes the contents of the internal data register to a block.



    NRSTPD = 22
    MAX_LEN = 16
    MI_OK       = 0
    MI_NOTAGERR = 1
    MI_ERR      = 2

    PCD_CALCCRC    = 0x03

    PICC_REQIDL    = 0x26
    PICC_REQALL    = 0x52
    PICC_ANTICOLL  = 0x93
    PICC_SElECTTAG = 0x93
    PICC_AUTHENT1A = 0x60
    PICC_AUTHENT1B = 0x61
    PICC_READ      = 0x30
    PICC_WRITE     = 0xA0
    PICC_DECREMENT = 0xC0
    PICC_INCREMENT = 0xC1
    PICC_RESTORE   = 0xC2
    PICC_TRANSFER  = 0xB0
    PICC_HALT      = 0x50



    # The commands used for MIFARE Ultralight (from http://www.nxp.com/documents/data_sheet/MF0ICU1.pdf, Section 8.6)
    # The PICC_CMD_MF_READ and PICC_CMD_MF_WRITE can also be used for MIFARE Ultralight.
    PICC_CMD_UL_WRITE        = 0xA2        #Writes one 4 byte page to the PICC.

    MF_ACK                    = 0xA                 # The MIFARE Classic uses a 4 bit ACK/NAK. Any other value than 0xA is NAK.
    MF_KEY_SIZE                = 6            # A Mifare Crypto1 key is 6 bytes.


    def __init__(self,I,cs='CS1'):
        self.cs=cs
        self.I = I
        self.I.SPI.set_parameters(2,1,1,0)
        if not self.reset():
            self.connected=False
            return None
        self.write(self.TModeReg, 0x80)
        self.write(self.TPrescalerReg, 0xA9)
        self.write(self.TReloadRegH, 0x03)
        self.write(self.TReloadRegL, 0xE8)

        self.write(self.TxASKReg, 0x40)
        self.write(self.ModeReg, 0x3D)

        #Enable the antenna
        self.enableAntenna()
        self.connected = True

    def MFRC522_Init(self):
        GPIO.output(self.NRSTPD, 1)
        self.MFRC522_Reset();
        self.write(self.TModeReg, 0x8D)
        self.write(self.TPrescalerReg, 0x3E)
        self.write(self.TReloadRegL, 30)
        self.write(self.TReloadRegH, 0)

        self.write(self.TxAutoReg, 0x40)
        self.write(self.ModeReg, 0x3D)
        self.AntennaOn()

    def enableAntenna(self):
        val = self.read(self.TxControlReg);
        if ((val & 0x03) != 0x03):
            self.write(self.TxControlReg, val | 0x03);

    def reset(self):
        self.write(self.CommandReg,self.PCD_SoftReset)
        s=time.time()
        while (self.read(self.CommandReg) & (1<<4)):
            print ('wait')
            time.sleep(0.1)
            if time.time() - s > .5: return False
        return True

    def write(self,register,val):
        self.I.SPI.set_cs(self.cs,0)
        ret = self.I.SPI.send16(((register&0x7F)<<8)|val)
        self.I.SPI.set_cs(self.cs,1)
        return ret&0xFF

    def read(self,register):
        self.I.SPI.set_cs(self.cs,0)
        ret = self.I.SPI.send16((register|0x80)<<8)
        self.I.SPI.set_cs(self.cs,1)
        return ret&0xFF

    def readMany(self,register,total):
        self.I.SPI.set_cs(self.cs,0)
        self.I.SPI.send8(register)
        vals = []
        for a in range(total-1):
            vals.append( I.SPI.send8(register) )
        vals.append( I.SPI.send8(0) )
        self.I.SPI.set_cs(self.cs,1)
        return vals


    def getStatus(self):
        return self.read(self.Status1Reg)

    def getVersion(self):
        ver = self.read(self.VersionReg)
        if ver==0x88: print ('Cloned version: Fudan Semiconductors')
        elif ver==0x90: print ('version 1.0')
        elif ver==0x91: print ('version 1.0')
        elif ver==0x92: print ('version 2.0')
        else: print ('Unknown version ',ver)
        return ver


    def SetBitMask(self, reg, mask):
        tmp = self.read(reg)
        self.write(reg, tmp | mask)

    def ClearBitMask(self, reg, mask):
        tmp = self.read(reg);
        self.write(reg, tmp & (~mask))

    def MFRC522_ToCard(self,command,sendData):
        returnedData = []
        backLen = 0
        status = self.MI_ERR
        irqEn = 0x00
        waitIRq = 0x00
        lastBits = None
        n = 0
        i = 0

        if command == self.PCD_MFAuthent:
            irqEn = 0x12
            waitIRq = 0x10
        if command == self.PCD_Transceive:
            irqEn = 0x77
            waitIRq = 0x30

        self.write(self.ComIEnReg, irqEn|0x80)
        self.ClearBitMask(self.ComIrqReg, 0x80)
        self.SetBitMask(self.FIFOLevelReg, 0x80)

        self.write(self.CommandReg, self.PCD_Idle);

        for a in sendData:
            self.write(self.FIFODataReg, a)
        self.write(self.CommandReg, command)

        if command == self.PCD_Transceive:
            self.SetBitMask(self.BitFramingReg, 0x80)

        i = 2000
        while True:
            n = self.read(self.ComIrqReg)
            i = i - 1
            if ~((i!=0) and ~(n&0x01) and ~(n&waitIRq)):
                break

        self.ClearBitMask(self.BitFramingReg, 0x80)

        if i != 0:
            if (self.read(self.ErrorReg) & 0x1B)==0x00:
                status = self.MI_OK

                if n & irqEn & 0x01:
                  status = self.MI_NOTAGERR

                if command == self.PCD_Transceive:
                  n = self.read(self.FIFOLevelReg)
                  lastBits = self.read(self.ControlReg) & 0x07
                  if lastBits != 0:
                    backLen = (n-1)*8 + lastBits
                  else:
                    backLen = n*8

                  if n == 0:
                    n = 1
                  if n > self.MAX_LEN:
                    n = self.MAX_LEN

                  i = 0
                  while i<n:
                    returnedData.append(self.read(self.FIFODataReg))
                    i = i + 1;
            else:
                status = self.MI_ERR
        return (status,returnedData,backLen)


    def MFRC522_Request(self, reqMode):
        status = None
        backBits = None
        TagType = []

        self.write(self.BitFramingReg, 0x07)

        TagType.append(reqMode);
        (status,returnedData,backBits) = self.MFRC522_ToCard(self.PCD_Transceive, TagType)

        if ((status != self.MI_OK) | (backBits != 0x10)):
            status = self.MI_ERR
        return (status,backBits)

    def MFRC522_Anticoll(self):
        returnedData = []
        serNumCheck = 0

        serNum = []

        self.write(self.BitFramingReg, 0x00)

        serNum.append(self.PICC_ANTICOLL)
        serNum.append(0x20)

        (status,returnedData,backBits) = self.MFRC522_ToCard(self.PCD_Transceive,serNum)

        if(status == self.MI_OK):
            i = 0
            if len(returnedData)==5:
                while i<4:
                    serNumCheck = serNumCheck ^ returnedData[i]
                    i = i + 1
                if serNumCheck != returnedData[i]:
                    status = self.MI_ERR
            else:
                status = self.MI_ERR

        return (status,returnedData)
  
    def CalulateCRC(self, pIndata):
        self.ClearBitMask(self.DivIrqReg, 0x04)
        self.SetBitMask(self.FIFOLevelReg, 0x80);
        for a in pIndata:
            self.write(self.FIFODataReg, a)
        self.write(self.CommandReg, self.PCD_CALCCRC)
        for i in range(0xFF):
            n = self.read(self.DivIrqReg)
            if (n&0x04):
                break
        pOutData = []
        pOutData.append(self.read(self.CRCResultRegL))
        pOutData.append(self.read(self.CRCResultRegH))
        return pOutData
  
    def MFRC522_SelectTag(self, serNum):
        returnedData = []
        buf = []
        buf.append(self.PICC_SElECTTAG)
        buf.append(0x70)
        i = 0
        while i<5:
            buf.append(serNum[i])
            i = i + 1
        pOut = self.CalulateCRC(buf)
        buf.append(pOut[0])
        buf.append(pOut[1])
        (status, returnedData, backLen) = self.MFRC522_ToCard(self.PCD_Transceive, buf)

        if (status == self.MI_OK) and (backLen == 0x18):
            return returnedData[0]
        else:
            return 0
  
    def MFRC522_Auth(self, authMode, BlockAddr, Sectorkey, serNum):
        buff = []
        # First byte should be the authMode (A or B)
        buff.append(authMode)
        # Second byte is the trailerBlock (usually 7)
        buff.append(BlockAddr)
        # Now we need to append the authKey which usually is 6 bytes of 0xFF
        i = 0
        while(i < len(Sectorkey)):
            buff.append(Sectorkey[i])
            i = i + 1
        i = 0

        # Next we append the first 4 bytes of the UID
        while(i < 4):
            buff.append(serNum[i])
            i = i +1

        # Now we start the authentication itself
        (status, returnedData, backLen) = self.MFRC522_ToCard(self.PCD_MFAuthent,buff)

        # Check if an error occurred
        if not(status == self.MI_OK):
            print ("AUTH ERROR!!")
        if not (self.read(self.Status2Reg) & 0x08) != 0:
            print ("AUTH ERROR(status2reg & 0x08) != 0")

        # Return the status
        return status

    def MFRC522_StopCrypto1(self):
        self.ClearBitMask(self.Status2Reg, 0x08)
        self.SetBitMask(self.CommandReg, 0x10)

    def MFRC522_Read(self, blockAddr):
        recvData = []
        recvData.append(self.PICC_READ)
        recvData.append(blockAddr)
        pOut = self.CalulateCRC(recvData)
        recvData.append(pOut[0])
        recvData.append(pOut[1])
        (status, returnedData, backLen) = self.MFRC522_ToCard(self.PCD_Transceive, recvData)
        if not(status == self.MI_OK):
            print ("Error while reading!")
        i = 0
        return returnedData

    def MFRC522_Write(self, blockAddr, writeData):
        buff = []
        buff.append(self.PICC_WRITE)
        buff.append(blockAddr)
        crc = self.CalulateCRC(buff)
        buff.append(crc[0])
        buff.append(crc[1])
        (status, returnedData, backLen) = self.MFRC522_ToCard(self.PCD_Transceive, buff)
        if not(status == self.MI_OK) or not(backLen == 4) or not((returnedData[0] & 0x0F) == 0x0A):
            status = self.MI_ERR

        print (str(backLen)+" returnedData &0x0F == 0x0A "+str(returnedData[0]&0x0F))
        if status == self.MI_OK:
            i = 0
            buf = []
            while i < 16:
                buf.append(writeData[i])
                i = i + 1
            crc = self.CalulateCRC(buf)
            buf.append(crc[0])
            buf.append(crc[1])
            (status, returnedData, backLen) = self.MFRC522_ToCard(self.PCD_Transceive,buf)
            if not(status == self.MI_OK) or not(backLen == 4) or not((returnedData[0] & 0x0F) == 0x0A):
                print ("Error while writing")
            if status == self.MI_OK:
                print ("Data written")

    def MFRC522_DumpClassic1K(self, key, uid):
        i = 0
        while i < 64:
            status = self.MFRC522_Auth(self.PICC_AUTHENT1A, i, key, uid)
            # Check if authenticated
            if status == self.MI_OK:
                self.MFRC522_Read(i)
            else:
                print ("Authentication error")
            i = i+1



if __name__ == "__main__":
    from expeyes import eyes17
    I= eyes17.open()
    A = MF522(I,'CS1')
    ret =  A.getStatus()
    print (ret,hex(ret),bin(ret))
    A.getVersion()
    import time
    while 1:
        (status,TagType) = A.MFRC522_Request(A.PICC_CMD_REQA)
        if status == A.MI_OK:
            print ("Card detected")
            (status,uid) = A.MFRC522_Anticoll()
            if status == A.MI_OK:
                print ("Card read UID: "+str(uid[0])+","+str(uid[1])+","+str(uid[2])+","+str(uid[3]))
                key = [0xFF,0xFF,0xFF,0xFF,0xFF,0xFF]
                A.MFRC522_SelectTag(uid)
                status = A.MFRC522_Auth(A.PICC_AUTHENT1A, 8, key, uid)
                if status == A.MI_OK:
                    A.MFRC522_Read(8)
                    # Variable for the data to write
                    '''
                    data = []
                    # Fill the data with 0xFF
                    for x in range(0,16):
                        data.append(0xFF)
                    print ("Sector 8 looked like this:")
                    # Read block 8
                    A.MFRC522_Read(8)
                    print ("\n")
                    print ("Sector 8 will now be filled with 0xFF:")
                    # Write the data
                    A.MFRC522_Write(8, data)
                    print ("\n")
                    print ("It now looks like this:")
                    # Check to see if it was written
                    A.MFRC522_Read(8)
                    print ("\n")
                    '''
                    A.MFRC522_StopCrypto1()
                else:
                    print ("Authentication error")
        else:
            print ('not detected')

#A.reset()







import sys, time
import functools
from functools import partial

if sys.version_info.major==3:
	from PyQt5 import QtGui, QtCore, QtWidgets
else:
	from PyQt4 import QtGui, QtCore
	from PyQt4 import QtGui as QtWidgets

import pyqtgraph as pg

import math, os.path, struct
import numpy as np
from collections import OrderedDict
from . import ui_advancedLogger,ui_inputSelector, ui_miniScope
from . gauge import Gauge
from . advancedLoggerTools import fit_sine,sine_eval,LOGGER, inputs, outputs

colors=['#00ff00','#ff0000','#ffff80',(10,255,255)]+[(50+np.random.randint(200),50+np.random.randint(200),150+np.random.randint(100)) for a in range(10)]

class miniscope(QtWidgets.QWidget,ui_miniScope.Ui_Form):
	tbvals = [0.100, 0.200, 0.500, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0,100.,200.]	# allowed mS/div values
	NP = 500			# Number of samples
	TG = 1				# Number of channels
	MINDEL = 1
	MAXDEL = 1000
	def __init__(self,parent,device):
		super(miniscope, self).__init__(parent)
		self.setupUi(self)
		self.p = device
		self.A1Box.addItems(self.p.allAnalogChannels)
		self.splitter.setSizes([500,100])
		self.activeParameter = 0

		self.curve = self.plot.plot(pen=colors[0])
		self.fitcurve = self.plot.plot(pen=colors[1],width=2)
		self.curve2 = self.plot.plot(pen=colors[2])
		self.fitcurve2 = self.plot.plot(pen=colors[3],width=2)

		self.region = pg.LinearRegionItem()
		self.region.setBrush([255,0,50,50])
		self.region.setZValue(10)
		for a in self.region.lines: a.setCursor(QtGui.QCursor(QtCore.Qt.SizeHorCursor)); 
		self.plot.addItem(self.region, ignoreBounds=False)
		self.region.setRegion([.1,.5])

		self.results = []
		for a in ['amplitude','frequency','phase','offset','','','','','','']:
			x = QtWidgets.QListWidgetItem(a)
			self.list.addItem(x)
			self.results.append(None)
	def changeParameter(self,p):
		self.activeParameter = p
		self.message.setText('Selected Parameter = %s of %s'%(self.list.item(p).text(),self.A1Box.currentText()))

	def timebaseChanged(self,val):
		msperdiv = self.tbvals[int(val)]				#millisecs / division
		totalusec = msperdiv * 1000 * 10.0  	# total 10 divisions
		self.TG = int(totalusec/self.NP)
		if self.TG < self.MINDEL:
			self.TG = self.MINDEL
		elif self.TG > self.MAXDEL:
			self.TG = self.MAXDEL
		xmax = self.TG*self.NP*1e-3
		self.plot.setRange(xRange=[0,xmax])
		self.timebaseLabel.setText('%.2f mS'%(totalusec/1e3))
		self.region.setRegion([.1*xmax,.95*xmax])

	def read(self,**kwargs):
		chan = str(self.A1Box.currentText())
		if chan in self.p.allAnalogChannels:
			for a in range(10): self.list.item(a).setText('')
			if self.A2Box.isChecked(): #2 channel capture
				t,v,t2,v2 = self.p.capture2(self.NP,self.TG,chan)
				self.curve.setData(t,v)
				kwargs['fitCurve'] = self.fitcurve
				res1 = self.sineFit(t,v,**kwargs)
	
				if res1 is not None:
					self.list.item(0).setText('Amplitude %.2f'%res1[0]);
					self.list.item(1).setText('Frequency %.2f'%res1[1])
					self.list.item(2).setText('Phase %.2f'%res1[2]);    
					self.list.item(3).setText('Offset %.2f'%res1[3]);


				self.curve2.setData(t2,v2)
				kwargs['fitCurve'] = self.fitcurve2
				res2 = self.sineFit(t2,v2,**kwargs)

				if res2 is not None:
					self.list.item(4).setText('Amplitude %.2f'%res2[0]);
					self.list.item(5).setText('Frequency %.2f'%res2[1])
					self.list.item(6).setText('Phase %.2f'%res2[2]);    
					self.list.item(7).setText('Offset %.2f'%res2[3]);

				if res1 is not None and res2 is not None:
					try:
						self.results = list(res1)+list(res2)
						self.results.append(res2[0]/res1[0]) #Amplitude ratio
						self.results.append(res2[2]-res1[2]) #Phase diff
						self.list.item(8).setText('AMP(2/1) %.2f'%(res2[0]/res1[0]))
						self.list.item(9).setText('Phase(2-1) %.2f'%(res2[2]-res1[2]))
						return float(self.results[self.activeParameter])
					except Exception as e:
						print(e)
						return False
			else:
				if self.activeParameter>3:
					self.activeParameter = 0
					self.list.setCurrentRow(0)
				t,v = self.p.capture1(chan,self.NP,self.TG)
				self.curve.setData(t,v)
				kwargs['fitCurve'] = self.fitcurve
				self.results = self.sineFit(t,v,**kwargs)

				if self.results is not None:
					self.list.item(0).setText('Amplitude %.2f'%self.results[0]);
					self.list.item(1).setText('Frequency %.2f'%self.results[1])
					self.list.item(2).setText('Phase %.2f'%self.results[2]);    
					self.list.item(3).setText('Offset %.2f'%self.results[3]);

					return float(self.results[self.activeParameter])
		return False

	def changeChannel(self,chan):
		chan = str(chan)
		miny = min(self.p.analogInputSources[chan].calPoly10(0),self.p.analogInputSources[chan].calPoly10(1023))
		maxy = max(self.p.analogInputSources[chan].calPoly10(0),self.p.analogInputSources[chan].calPoly10(1023))
		self.plot.setRange(yRange=[miny,maxy])

	def sineFit(self,t,v,**kwargs):
		S,E=self.region.getRegion()
		start = (np.abs(t - S)).argmin()
		end = (np.abs(t - E)).argmin()
		try:
			fa=fit_sine(t[start:end],v[start:end])
			if fa is not None:
					amp=abs(fa[0])
					freq=fa[1]*1e3
					if 'freq' in kwargs and self.freqCheckBox.isChecked(): #Input frequency supplied. check for fitting error
						if ( abs(kwargs.get('freq') - freq)/freq ) > 0.1: #Frequency mismatch >10%
								return None
							 
					phase = fa[2]*180/np.pi
					offset = fa[3]
					#self.message.setText(s)
					x = np.linspace(t[start],t[end],1000)
					if 'fitCurve' in kwargs:
						kwargs['fitCurve'].clear()
						kwargs['fitCurve'].setData(x,sine_eval(x,fa))
					return amp,freq,phase,offset	
		except Exception as e:
			self.message.setText('fit failed')
			print(e)
		return None

class DIOINPUT(QtWidgets.QDialog,ui_inputSelector.Ui_Dialog):
	SLIDER_SCALING = 1000.
	def __init__(self,parent,device,confirmValues):
		super(DIOINPUT, self).__init__(parent)
		self.setupUi(self)
		self.confirmValues = confirmValues
		self.subSelection.setStyleSheet("border: 3px dashed #5353ff;")
		self.selectedGauge = None

		self.p = device
		self.I2C = self.p.I2C
		self.inputs = inputs(self.p)
		self.outputs = outputs(self.p)
		self.type = None
		self.autoRefresh = True
		self.functions = []

		
		self.initialize = None
		self.read = None
		self.widgets =[]
		self.gauges = []
		self.miniscope = None
		self.activeSensor = None
		self.permanentInputs = self.inputs.permanentInputs
		self.permanentOutputs = self.outputs.permanentOutputs
		self.init()
		
	def init(self):
		self.updateOptions(self.permanentInputs+self.refreshSensorList(),self.permanentOutputs)

	def refreshSensorList(self):
		self.logger = LOGGER(self.I2C)
		x = self.logger.I2CScan()
		print('Responses from: ',x)
		self.sensorList = []
		for a in x:
			s = self.logger.sensors.get(a,None)
			if s is not None:
				self.sensorList.append(s)
		return self.sensorList

	def updateOptions(self,sensors,outputs):
		self.sensors = sensors+outputs
		self.availableInputs.clear()
		self.availableInputs.addItems([a['name'] for a in self.sensors])
		if self.activeSensor not in self.sensors:
			self.loadSensor(self.sensors[0])

	def selectSensor(self,index):
		self.loadSensor(self.sensors[index])
		self.subSelectionChanged(0)

	def subSelectionChanged(self,index):
		name = str(self.subSelection.currentText())
		self.subSelectionIndex = index
		for a in self.gauges:
			if a.title_text == name:
				a.gauge_color_inner_radius_factor = 0.5
				a.set_NeedleColor(255, 0, 0, 255)
				self.selectedGauge = a
			else:
				a.gauge_color_inner_radius_factor = 0.9
				a.set_NeedleColor(100, 100, 100, 255)
		self.minValue.setValue(self.activeSensor['min'][index])
		self.maxValue.setValue(self.activeSensor['max'][index])

	def loadSensor(self,sensor):
		self.activeSensor = sensor
		self.name = sensor['name']
		self.funtions = {} 
		self.initialize = sensor['init']
		self.initialize()


		self.max = sensor.get('max',None)
		self.min = sensor.get('min',None)
		self.type = sensor['type']
		self.autoRefresh = sensor.get('autorefresh',True)

		for a in self.widgets:
				a.setParent(None)
		self.widgets =[]
		for a in sensor.get('config',[]): #Load configuration menus
			l = QtWidgets.QLabel(a.get('name',''))
			self.configLayout.addWidget(l) ; self.widgets.append(l)
			l = QtWidgets.QComboBox(); l.addItems(a.get('options',[]))
			l.currentIndexChanged['int'].connect(a.get('function',None))
			self.configLayout.addWidget(l) ; self.widgets.append(l)
			
		for a in self.gauges:
				a.setParent(None)
		self.gauges = []
		self.subSelection.clear()

		if self.miniscope:
			self.miniscope.setParent(None);
			self.read=None;
			self.miniscope=None

		self.functions = []
		row = 1; col=1;
		parameters=0

		if 'scope' in self.name: # It's an oscilloscope. make a plot instead of gauges
			self.miniscope = miniscope(self,self.p)
			self.gaugeLayout.addWidget(self.miniscope)
			self.setWindowTitle('Oscilloscope with analysis')
			self.read = self.miniscope.read

		else:
			self.fields = sensor.get('fields',None)
			self.subSelection.addItems(self.fields)
			self.subSelectionIndex = 0
			for a,b,c in zip(self.fields,self.min,self.max):
				gauge = Gauge(self,a)
				gauge.setObjectName(a)
				gauge.set_MinValue(b)
				gauge.set_MaxValue(c)
				self.gaugeLayout.addWidget(gauge,row,col)
				self.gauges.append(gauge)
				col+= 1
				if col == 4:
					row += 1
					col = 1


				if sensor['type'] == 'output':
					l = QtWidgets.QSlider(self); l.setMinimum(b*self.SLIDER_SCALING); l.setMaximum(c*self.SLIDER_SCALING);l.setValue(b*self.SLIDER_SCALING);
					l.setOrientation(QtCore.Qt.Horizontal)
					gauge.value_needle_snapzone = 1
					gauge.valueChanged.connect(functools.partial(self.showval,parameters))

					l.valueChanged['int'].connect(functools.partial(self.write,parameters))
					self.configLayout.addWidget(l) ; self.widgets.append(l)
					self.functions.append(sensor['write'])
				parameters+=1

			if not self.autoRefresh: #Time consuming , blocking function call. add a button for it.
				l = QtWidgets.QPushButton("MAKE A MEASUREMENT",self)
				l.clicked.connect(self.readAndUpdate)
				self.configLayout.addWidget(l) ; self.widgets.append(l)

			if sensor['type'] == 'input':
				self.read = sensor['read']
				self.setWindowTitle('Sensor : %s'%self.name)
			else:
				self.read = None
				self.setWindowTitle('Output : %s'%self.name)

	def showval(self,index,v):
		self.gauges[index].value = v
		self.gauges[index].update()
		self.widgets[index].setValue(v*self.SLIDER_SCALING)

	def write(self,index,val):
		val/=self.SLIDER_SCALING
		self.gauges[index].update_value(val)
		self.functions[index](val)

	def readAndUpdate(self):
		a = self.read()
		self.message.setText(str(a))
		if a is not None:
			self.setValue(a)

	def setValue(self,vals):
		if vals is None:
			print('check connections')
			return
		p=0
		for a in self.gauges:
			a.update_value(vals[p])
			p+=1

	def confirm(self):
		if self.confirmValues is None: return
		if 'scope' in self.name:
			self.confirmValues('Oscilloscope:%s:%s'%(self.miniscope.A1Box.currentText(),self.miniscope.list.item(self.miniscope.activeParameter).text()))
		else:
			self.confirmValues('%s:%s'%(self.activeSensor['name'],self.activeSensor['fields'][self.subSelectionIndex]))

		self.hide()

	def initSweep(self,steps):
		self.value = self.minValue.value()#self.min[self.subSelectionIndex]
		self.endValue = self.maxValue.value()#self.max[self.subSelectionIndex]
		self.stepSize = (self.endValue - self.value)/steps
		self.message.setText('%.2f -> %.2f in %d steps'%(self.value,self.endValue,steps)) 
		if self.type == 'output': #Output
			self.write(self.subSelectionIndex,self.value*self.SLIDER_SCALING)
			self.message.setText('%.2f / %.2f'%(self.value,self.endValue))
	
	def nextValue(self,**kwargs):
		if 'scope' in self.name:
			return self.read(**kwargs)
		elif self.type == 'input':
			a = self.read()
			if a is not None:
				if len(a)>=self.subSelectionIndex:
					self.setValue(a)
					try:
						return a[self.subSelectionIndex]
					except:
						return False
		elif self.type == 'output': #Output
			self.write(self.subSelectionIndex,self.value*self.SLIDER_SCALING)
			self.message.setText('%.2f / %.2f'%(self.value,self.endValue))
			v = self.value

			self.value+=self.stepSize
			if self.value>self.endValue:
				return None #None returned will stop the acquisition
			return v

		return False

	def getValue(self,a):
		v = a.read()
		if v is not None:
			return v[self.subSelectionIndex]
		return None

	def launch(self,setWindow=None):
		if self.initialize is not None:
			self.initialize()
		if setWindow is not None:
			p=0
			for a in self.sensors:
				if setWindow == a['name']:
					#self.loadSensor(a)
					self.availableInputs.setCurrentIndex(p)
					break
				p+=1
		self.show()






# -*- coding: utf-8; mode: python; indent-tabs-mode: t; tab-width:4 -*-
import sys, time, utils, math, os.path

from QtVersion import *

import pyqtgraph as pg
import numpy as np
import eyes17.eyemath17 as em


class Expt(QWidget):
	TIMER = 50
	RPWIDTH = 300
	RPGAP = 4
	running = False
	
	VMIN = 0
	VMAX = -5.0
	VSET = VMIN
	IMIN = 0
	IMAX = -5.0
	STEP = -0.050	   # 50 mV
	data = [ [], [] ]
	currentTrace = None
	traces = []
	legends = []
	history = []		# Data store	
	
	def __init__(self, device=None):
		QWidget.__init__(self)
		self.p = device										# connection to the device hardware 

		self.traceCols = utils.makeTraceColors()
		self.resultCols = utils.makeResultColors()
		self.trial = 0
		
		self.pwin = pg.PlotWidget()							# pyqtgraph window
		self.pwin.showGrid(x=True, y=True)					# with grid
		ax = self.pwin.getAxis('bottom')
		ax.setLabel(self.tr('Voltage (V)'))	
		ax = self.pwin.getAxis('left')
		ax.setLabel(self.tr('Current (mA)'))
		self.pwin.disableAutoRange()
		self.pwin.setXRange(self.VMIN, self.VMAX)
		self.pwin.setYRange(self.IMIN, self.IMAX)
		self.pwin.hideButtons()								# Do not show the 'A' button of pg

		right = QVBoxLayout()							# right side vertical layout
		right.setAlignment(Qt.AlignTop)
		right.setSpacing(self.RPGAP)
		
		H = QHBoxLayout()
		l = QLabel(text=self.tr('Vbase (via 100kOhm)'))
		l.setMaximumWidth(140)
		H.addWidget(l)
		self.VBtext = utils.lineEdit(40, -1.0, 6, None)
		H.addWidget(self.VBtext)
		l = QLabel(text=self.tr('V'))
		l.setMaximumWidth(10)
		H.addWidget(l)
		right.addLayout(H)
		 
		b = QPushButton(self.tr("Start"))
		right.addWidget(b)
		b.clicked.connect(self.start)		
		
		b = QPushButton(self.tr("Stop"))
		right.addWidget(b)
		b.clicked.connect(self.stop)		
		
		b = QPushButton(self.tr("Clear Traces"))
		right.addWidget(b)
		b.clicked.connect(self.clear)		

		self.SaveButton = QPushButton(self.tr("Save Data"))
		self.SaveButton.clicked.connect(self.save_data)		
		right.addWidget(self.SaveButton)

		#------------------------end of right panel ----------------
		
		top = QHBoxLayout()
		top.addWidget(self.pwin)
		top.addLayout(right)
		
		full = QVBoxLayout()
		full.addLayout(top)
		self.msgwin = QLabel(text=self.tr(''))
		full.addWidget(self.msgwin)
				
		self.setLayout(full)
		
		self.timer = QTimer()
		self.timer.timeout.connect(self.update)
		self.timer.start(self.TIMER)
		#----------------------------- end of init ---------------

				
	def update(self):
		if self.running == False:
			return
		try:
			vs = self.p.set_pv1(self.VSET)	
			time.sleep(0.001)	
			va = self.p.get_voltage('A1')		# voltage across the diode
		except:
			self.comerr()
			return 

		i = (vs-va)/1.0 	 		   # in mA, R= 1k
		self.data[0].append(va)
		self.data[1].append(i)
		self.VSET += self.STEP
		if self.VSET < self.VMAX:
			self.running = False
			self.history.append(self.data)
			self.traces.append(self.currentTrace)
			self.msg(self.tr('Completed plotting I-V'))
			l = pg.TextItem(text=self.ibtxt, color= self.resultCols[self.trial])
			self.trial += 1
			l.setPos(va,i)
			self.pwin.addItem(l)
			self.legends.append(l)
			return
		if self.index > 1:			  # Draw the line
			self.currentTrace.setData(self.data[0], self.data[1])
		self.index += 1


	def start(self):
		if self.running == True: 
			return
		text = self.VBtext.text()
		try:
			vbset = float(text)
			if vbset > -0.5 or vbset < -3.0:
				self.msg(self.tr('Base valtage shold be from -0.5 to -3'))
				return
		except:
			self.msg(self.tr('Invalid Base valtage, shold be from -0.5 to -3'))
			return
				
		try:
			self.p.set_pv1(-5.0)			# Collector to 5V
			self.p.set_pv2(vbset)		# Set base bias on PV2, via 100 KOhm series resistance
			vb = self.p.get_voltage('A2')    # base voltage
		except:
			self.comerr()
			return 

		if vb > -0.5 or vb < -0.7:
			vb = -0.6
		ibase = (vbset-vb)/100.0e-3    # uA
		self.ibtxt = 'Ib = %5.3f uA'%ibase
		self.running = True
		self.data = [ [], [] ]
		self.VSET = self.VMIN
		self.currentTrace = self.pwin.plot([0,0],[0,0], pen = self.traceCols[self.trial%5])
		self.index = 0
		self.msg(self.tr('Started'))

	def stop(self):
		if self.running == False: return
		self.running = False
		self.history.append(self.data)
		self.traces.append(self.currentTrace)
		self.msg(self.tr('User Stopped'))

	def clear(self):
		for k in self.legends:
			self.pwin.removeItem(k)
		for k in self.traces:
			self.pwin.removeItem(k)
		self.history = []
		self.trial = 0
		self.msg(self.tr('Cleared Traces and Data'))

	def save_data(self):
		if self.history == []:
			self.msg(self.tr('No data to save'))
			return
		fn = QFileDialog.getSaveFileName()
		if fn != '':
			self.p.save(self.history, fn)
			self.msg(self.tr('Traces saved to ') + unicode(fn))				
		
	def msg(self, m):
		self.msgwin.setText(self.tr(m))
		
	def comerr(self):
		self.msgwin.setText('<font color="red">' + self.tr('Error. Try Device->Reconnect'))

if __name__ == '__main__':
	import eyes17.eyes
	dev = eyes17.eyes.open()
	app = QApplication(sys.argv)

	# translation stuff
	lang=QLocale.system().name()
	t=QTranslator()
	t.load("lang/"+lang, os.path.dirname(__file__))
	app.installTranslator(t)
	t1=QTranslator()
	t1.load("qt_"+lang,
	        QLibraryInfo.location(QLibraryInfo.TranslationsPath))
	app.installTranslator(t1)

	mw = Expt(dev)
	mw.show()
	sys.exit(app.exec_())
	



# -*- coding: utf-8; mode: python; indent-tabs-mode: t; tab-width:4 -*-
'''
Code for science experiments using expEYES-17 interface
Author  : Ajith Kumar B.P, bpajith@gmail.com
Date    : Aug-2017
License : GNU GPL version 3
'''

import sys, time, utils, math, os.path

from QtVersion import *

import pyqtgraph as pg
import numpy as np
import eyes17.eyemath17 as em


class Expt(QWidget):
	TIMER = 5
	RPWIDTH = 300
	RPGAP = 4
	running = False
	
	TPMIN = 0
	TPMAX = 100
	TMIN = 0
	TMAX = 10
	TGAP = 1.0
	Gain = 11.0
	CCval = 1.1
	Offset = 0.0
	data = [ [], [] ]
	currentTrace = None
	traces = []
	history = []		# Data store	
	sources = ['A1','A2','A3', 'MIC']
	trial = 0
	
	def __init__(self, device=None):
		QWidget.__init__(self)
		self.p = device										# connection to the device hardware 
		
		try:
			self.p.set_state(CCS=1)
		except:
			pass	
		self.traceCols = utils.makeTraceColors()
		
		self.pwin = pg.PlotWidget()							# pyqtgraph window
		self.pwin.showGrid(x=True, y=True)					# with grid
		ax = self.pwin.getAxis('bottom')
		ax.setLabel(self.tr('Time (mS)'))
		ax = self.pwin.getAxis('left')
		ax.setLabel(self.tr('Temparature (C)'))
		self.pwin.disableAutoRange()
		self.pwin.setXRange(self.TMIN, self.TMAX)
		self.pwin.setYRange(self.TPMIN, self.TPMAX)
		self.pwin.hideButtons()								# Do not show the 'A' button of pg

		right = QVBoxLayout()							# right side vertical layout
		right.setAlignment(Qt.AlignTop)
		right.setSpacing(self.RPGAP)

		H = QHBoxLayout()
		b = QPushButton(self.tr("Measure A3"))
		b.setMaximumWidth(120)
		H.addWidget(b)
		b.clicked.connect(self.measureA3)		
		self.A3val = QLabel(text='')
		H.addWidget(self.A3val)
		right.addLayout(H)


		H = QHBoxLayout()
		l = QLabel(text=self.tr('A3 Gain'))
		l.setMaximumWidth(100)
		H.addWidget(l)
		self.GAINtext = utils.lineEdit(40, self.Gain, 6, None)
		H.addWidget(self.GAINtext)
		l = QLabel(text=self.tr('1+10k/Rg'))
		l.setMaximumWidth(60)
		H.addWidget(l)
		right.addLayout(H)

		H = QHBoxLayout()
		l = QLabel(text=self.tr('A3 Offset'))
		l.setMaximumWidth(100)
		H.addWidget(l)
		self.OFFSETtext = utils.lineEdit(40, self.Offset, 6, None)
		H.addWidget(self.OFFSETtext)
		l = QLabel(text=self.tr('mV '))
		l.setMaximumWidth(60)
		H.addWidget(l)
		right.addLayout(H)

		H = QHBoxLayout()
		l = QLabel(text=self.tr('CCS Value'))
		l.setMaximumWidth(100)
		H.addWidget(l)
		self.CCtext = utils.lineEdit(40, self.CCval, 6, None)
		H.addWidget(self.CCtext)
		l = QLabel(text=self.tr('mA'))
		l.setMaximumWidth(60)
		H.addWidget(l)
		right.addLayout(H)
		
					
		H = QHBoxLayout()
		l = QLabel(text=self.tr('Lowest Temp'))
		l.setMaximumWidth(100)
		H.addWidget(l)
		self.TPMINtext = utils.lineEdit(40, self.TPMIN, 6, None)
		H.addWidget(self.TPMINtext)
		l = QLabel(text=self.tr('deg C'))
		l.setMaximumWidth(60)
		H.addWidget(l)
		right.addLayout(H)

		H = QHBoxLayout()
		l = QLabel(text=self.tr('Highest Temp'))
		l.setMaximumWidth(100)
		H.addWidget(l)
		self.TPMAXtext = utils.lineEdit(40, self.TPMAX, 6, None)
		H.addWidget(self.TPMAXtext)
		l = QLabel(text=self.tr('deg C'))
		l.setMaximumWidth(60)
		H.addWidget(l)
		right.addLayout(H)
		
		H = QHBoxLayout()
		l = QLabel(text=self.tr('Total Duration'))
		l.setMaximumWidth(100)
		H.addWidget(l)
		self.TMAXtext = utils.lineEdit(40, self.TMAX, 6, None)
		H.addWidget(self.TMAXtext)
		l = QLabel(text=self.tr('Seconds'))
		l.setMaximumWidth(60)
		H.addWidget(l)
		right.addLayout(H)

		H = QHBoxLayout()
		l = QLabel(text=self.tr('Measure every'))
		l.setMaximumWidth(100)
		H.addWidget(l)
		self.TGAPtext = utils.lineEdit(40, self.TGAP, 6, None)
		H.addWidget(self.TGAPtext)
		l = QLabel(text=self.tr('Seconds'))
		l.setMaximumWidth(60)
		H.addWidget(l)
		right.addLayout(H)

		b = QPushButton(self.tr("Start"))
		right.addWidget(b)
		b.clicked.connect(self.start)		
		
		b = QPushButton(self.tr("Stop"))
		right.addWidget(b)
		b.clicked.connect(self.stop)		
		
		b = QPushButton(self.tr("Clear Traces"))
		right.addWidget(b)
		b.clicked.connect(self.clear)		

		self.SaveButton = QPushButton(self.tr("Save Data"))
		self.SaveButton.clicked.connect(self.save_data)		
		right.addWidget(self.SaveButton)

		#------------------------end of right panel ----------------
		
		top = QHBoxLayout()
		top.addWidget(self.pwin)
		top.addLayout(right)
		
		full = QVBoxLayout()
		full.addLayout(top)
		self.msgwin = QLabel(text='')
		full.addWidget(self.msgwin)
				
		self.setLayout(full)
		
		self.timer = QTimer()
		self.timer.timeout.connect(self.update)
		self.timer.start(self.TIMER)
		

		#----------------------------- end of init ---------------
	

	def v2t(self, v):					# Convert Voltage to Temperature for PT100
		v = v - self.Offset* 0.001      # Convert Offset to volts
		vr = v/self.Gain    		    # voltage across PT100
		r = vr / (self.CCval * 1.0e-3)  # mA to Ampere
		r0 = 100.0						# PT100 parameters r0, A and B 
		A = 3.9083e-3
		B = -5.7750e-7
		c = 1 - r/r0
		b4ac = math.sqrt( A*A - 4 * B * c)
		temp = (-A + b4ac) / (2.0 * B)
		return temp

	def measureA3(self):
		try:	
			sum = 0.0
			NT = 4
			for k in range(NT):
				v = self.p.get_voltage('A3')  		# Read A3
				sum += v
			v = sum/NT
			self.A3val.setText(unicode(self.tr('%5.3f V')) %v)
		except:
			self.comerr()
			return 
			
	def update(self):
		if self.running == False:
			return
		try:	
			sum = 0.0
			NT = 4
			for k in range(NT):
				t,v = self.p.get_voltage_time('A3')  		# Read A3
				sum += v
			v = sum/NT
		except:
			self.comerr()
			return 

		if len(self.data[0]) == 0:
			self.start_time = t
			elapsed = 0
		else:
			elapsed = t - self.start_time

		self.data[0].append(elapsed)
		temp = self.v2t(v)
		self.data[1].append(temp)
		if elapsed > self.TMAX:
			self.running = False
			self.history.append(self.data)
			self.traces.append(self.currentTrace)
			self.msg(self.tr('Time Vs Temperature plot completed'))
			return
		if self.index > 1:			  # Draw the line
			self.currentTrace.setData(self.data[0], self.data[1])
		self.index += 1


	def start(self):
		if self.running == True: return
		try:
			self.TMAX = float(self.TMAXtext.text())
			self.TPMAX = float(self.TPMAXtext.text())
		except:
			self.msg(self.tr('Invalid Duration'))
			return
			
		try:
			self.TGAP = float(self.TGAPtext.text())
			self.TIMER = self.TGAP * 1000    # into mS
			self.timer.stop()
			self.timer.start(self.TIMER)
		except:
			self.msg(self.tr('Invalid time interval between reads'))
			return

		try:
			self.TPMIN = float(self.TPMINtext.text())
			self.TPMAX = float(self.TPMAXtext.text())
		except:
			self.msg(self.tr('Invalid temperature limit'))
			return
			
		try:
			self.Offset = float(self.OFFSETtext.text())
			self.Gain = float(self.GAINtext.text())
		except:
			self.msg(self.tr('Invalid Offset or Gain'))
			return

		try:
			self.CCval = float(self.CCtext.text())
		except:
			self.msg(self.tr('Invalid CCS input'))
			return
		
		self.pwin.setXRange(self.TMIN, self.TMAX)
		self.pwin.setYRange(self.TPMIN, self.TPMAX)
		self.running = True
		self.data = [ [], [] ]
		self.currentTrace = self.pwin.plot([0,0],[0,0], pen = self.traceCols[self.trial%5])
		self.index = 0
		self.trial += 1
		self.msg(self.tr('Started Measurements'))

	def stop(self):
		if self.running == False: return
		self.running = False
		self.history.append(self.data)
		self.traces.append(self.currentTrace)
		self.msg(self.tr('User Stopped'))

	def clear(self):
		if self.running == True: return
		for k in self.traces:
			self.pwin.removeItem(k)
		self.history = []
		self.trial = 0
		self.msg(self.tr('Cleared Traces and Data'))
		
	def save_data(self):
		if self.history == []:
			self.msg(self.tr('No data to save'))
			return
		fn = QFileDialog.getSaveFileName()
		if fn != '':
			self.p.save(self.history, fn)
			self.msg(self.tr('Traces saved to ') + unicode(fn))				
		
	def msg(self, m):
		self.msgwin.setText(self.tr(m))
		
	def comerr(self):
		self.msgwin.setText('<font color="red">' + self.tr('Error. Try Device->Reconnect'))

if __name__ == '__main__':
	import eyes17.eyes
	dev = eyes17.eyes.open()
	
	dev.set_pv1(0.120)
	dev.set_pv2(0.120)
	app = QApplication(sys.argv)

	# translation stuff
	lang=QLocale.system().name()
	t=QTranslator()
	t.load("lang/"+lang, os.path.dirname(__file__))
	app.installTranslator(t)
	t1=QTranslator()
	t1.load("qt_"+lang,
	        QLibraryInfo.location(QLibraryInfo.TranslationsPath))
	app.installTranslator(t1)

	mw = Expt(dev)
	mw.show()
	sys.exit(app.exec_())
	



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'scope_layout.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(876, 507)
        Form.setStyleSheet("QSlider::groove:vertical{\n"
"    border: 1px solid #565a5e;\n"
"    width: 10px;\n"
"    background: #eee;\n"
"    margin: 0px;\n"
"    border-radius: 4px;\n"
"}\n"
"QSlider::handle:vertical{\n"
"    border: 1px solid #565a5e;\n"
"    width: 5px;\n"
"    height: 10px;\n"
"    border-radius: 4px;\n"
"}")
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.Diff = QtWidgets.QCheckBox(Form)
        self.Diff.setMaximumSize(QtCore.QSize(60, 16777215))
        self.Diff.setObjectName("Diff")
        self.gridLayout.addWidget(self.Diff, 1, 3, 1, 1)
        self.msgwin = QtWidgets.QLabel(Form)
        self.msgwin.setObjectName("msgwin")
        self.gridLayout.addWidget(self.msgwin, 1, 0, 1, 2)
        self.splitter = QtWidgets.QSplitter(Form)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.pwin = PlotWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(90)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pwin.sizePolicy().hasHeightForWidth())
        self.pwin.setSizePolicy(sizePolicy)
        self.pwin.setObjectName("pwin")
        self.frame = QtWidgets.QFrame(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.frame)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 5)
        self.gridLayout_2.setSpacing(2)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_4 = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setObjectName("label_4")
        self.gridLayout_2.addWidget(self.label_4, 0, 1, 1, 3)
        self.frame_4 = QtWidgets.QFrame(self.frame)
        self.frame_4.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_4.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_4.setObjectName("frame_4")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.frame_4)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setSpacing(2)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.A2Range = QtWidgets.QComboBox(self.frame_4)
        self.A2Range.setMaximumSize(QtCore.QSize(50, 16777215))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.A2Range.setFont(font)
        self.A2Range.setObjectName("A2Range")
        self.A2Range.addItem("")
        self.A2Range.addItem("")
        self.A2Range.addItem("")
        self.A2Range.addItem("")
        self.A2Range.addItem("")
        self.A2Range.addItem("")
        self.gridLayout_4.addWidget(self.A2Range, 1, 2, 1, 1)
        self.MICFit = QtWidgets.QCheckBox(self.frame_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.MICFit.sizePolicy().hasHeightForWidth())
        self.MICFit.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.MICFit.setFont(font)
        self.MICFit.setText("")
        self.MICFit.setObjectName("MICFit")
        self.gridLayout_4.addWidget(self.MICFit, 3, 3, 1, 1)
        self.A1Map = QtWidgets.QComboBox(self.frame_4)
        self.A1Map.setMaximumSize(QtCore.QSize(55, 16777215))
        self.A1Map.setObjectName("A1Map")
        self.gridLayout_4.addWidget(self.A1Map, 0, 1, 1, 1)
        self.A3Fit = QtWidgets.QCheckBox(self.frame_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.A3Fit.sizePolicy().hasHeightForWidth())
        self.A3Fit.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.A3Fit.setFont(font)
        self.A3Fit.setText("")
        self.A3Fit.setObjectName("A3Fit")
        self.gridLayout_4.addWidget(self.A3Fit, 2, 3, 1, 1)
        self.A3Box = QtWidgets.QCheckBox(self.frame_4)
        self.A3Box.setObjectName("A3Box")
        self.gridLayout_4.addWidget(self.A3Box, 2, 0, 1, 2)
        self.A1Range = QtWidgets.QComboBox(self.frame_4)
        self.A1Range.setMaximumSize(QtCore.QSize(50, 16777215))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.A1Range.setFont(font)
        self.A1Range.setObjectName("A1Range")
        self.A1Range.addItem("")
        self.A1Range.addItem("")
        self.A1Range.addItem("")
        self.A1Range.addItem("")
        self.A1Range.addItem("")
        self.A1Range.addItem("")
        self.gridLayout_4.addWidget(self.A1Range, 0, 2, 1, 1)
        self.A2Box = QtWidgets.QCheckBox(self.frame_4)
        self.A2Box.setObjectName("A2Box")
        self.gridLayout_4.addWidget(self.A2Box, 1, 0, 1, 2)
        self.A1Fit = QtWidgets.QCheckBox(self.frame_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.A1Fit.sizePolicy().hasHeightForWidth())
        self.A1Fit.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.A1Fit.setFont(font)
        self.A1Fit.setText("")
        self.A1Fit.setObjectName("A1Fit")
        self.gridLayout_4.addWidget(self.A1Fit, 0, 3, 1, 1)
        self.A2Fit = QtWidgets.QCheckBox(self.frame_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.A2Fit.sizePolicy().hasHeightForWidth())
        self.A2Fit.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.A2Fit.setFont(font)
        self.A2Fit.setText("")
        self.A2Fit.setObjectName("A2Fit")
        self.gridLayout_4.addWidget(self.A2Fit, 1, 3, 1, 1)
        self.A1Box = QtWidgets.QCheckBox(self.frame_4)
        self.A1Box.setText("")
        self.A1Box.setObjectName("A1Box")
        self.gridLayout_4.addWidget(self.A1Box, 0, 0, 1, 1)
        self.A3Range = QtWidgets.QComboBox(self.frame_4)
        self.A3Range.setMaximumSize(QtCore.QSize(50, 16777215))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.A3Range.setFont(font)
        self.A3Range.setObjectName("A3Range")
        self.A3Range.addItem("")
        self.A3Range.addItem("")
        self.A3Range.addItem("")
        self.A3Range.addItem("")
        self.gridLayout_4.addWidget(self.A3Range, 2, 2, 1, 1)
        self.MICRange = QtWidgets.QComboBox(self.frame_4)
        self.MICRange.setMaximumSize(QtCore.QSize(50, 16777215))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.MICRange.setFont(font)
        self.MICRange.setObjectName("MICRange")
        self.MICRange.addItem("")
        self.MICRange.addItem("")
        self.MICRange.addItem("")
        self.MICRange.addItem("")
        self.gridLayout_4.addWidget(self.MICRange, 3, 2, 1, 1)
        self.MICBox = QtWidgets.QCheckBox(self.frame_4)
        self.MICBox.setObjectName("MICBox")
        self.gridLayout_4.addWidget(self.MICBox, 3, 0, 1, 2)
        self.gridLayout_2.addWidget(self.frame_4, 13, 1, 1, 4)
        self.frame_3 = QtWidgets.QFrame(self.frame)
        self.frame_3.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_3.setObjectName("frame_3")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.frame_3)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setSpacing(1)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.PV2text = QtWidgets.QDoubleSpinBox(self.frame_3)
        self.PV2text.setMaximumSize(QtCore.QSize(150, 16777215))
        self.PV2text.setMinimum(-3.0)
        self.PV2text.setMaximum(3.0)
        self.PV2text.setObjectName("PV2text")
        self.gridLayout_3.addWidget(self.PV2text, 3, 3, 1, 2)
        self.SQ1DCtext = QtWidgets.QSpinBox(self.frame_3)
        self.SQ1DCtext.setMaximumSize(QtCore.QSize(55, 16777215))
        self.SQ1DCtext.setProperty("value", 50)
        self.SQ1DCtext.setObjectName("SQ1DCtext")
        self.gridLayout_3.addWidget(self.SQ1DCtext, 1, 4, 1, 1)
        self.SQ1slider = QtWidgets.QSlider(self.frame_3)
        self.SQ1slider.setMinimum(0)
        self.SQ1slider.setMaximum(50000)
        self.SQ1slider.setSliderPosition(1000)
        self.SQ1slider.setOrientation(QtCore.Qt.Horizontal)
        self.SQ1slider.setObjectName("SQ1slider")
        self.gridLayout_3.addWidget(self.SQ1slider, 1, 1, 1, 1)
        self.SQ1text = QtWidgets.QDoubleSpinBox(self.frame_3)
        self.SQ1text.setMaximumSize(QtCore.QSize(80, 16777215))
        self.SQ1text.setMaximum(50000.0)
        self.SQ1text.setObjectName("SQ1text")
        self.gridLayout_3.addWidget(self.SQ1text, 1, 3, 1, 1)
        self.PV1slider = QtWidgets.QSlider(self.frame_3)
        self.PV1slider.setMinimum(-5000)
        self.PV1slider.setMaximum(5000)
        self.PV1slider.setProperty("value", 0)
        self.PV1slider.setOrientation(QtCore.Qt.Horizontal)
        self.PV1slider.setObjectName("PV1slider")
        self.gridLayout_3.addWidget(self.PV1slider, 2, 1, 1, 2)
        self.PV2slider = QtWidgets.QSlider(self.frame_3)
        self.PV2slider.setMinimum(-3300)
        self.PV2slider.setMaximum(3300)
        self.PV2slider.setOrientation(QtCore.Qt.Horizontal)
        self.PV2slider.setObjectName("PV2slider")
        self.gridLayout_3.addWidget(self.PV2slider, 3, 1, 1, 2)
        self.AWGslider = QtWidgets.QSlider(self.frame_3)
        self.AWGslider.setMinimum(5)
        self.AWGslider.setMaximum(5000)
        self.AWGslider.setOrientation(QtCore.Qt.Horizontal)
        self.AWGslider.setObjectName("AWGslider")
        self.gridLayout_3.addWidget(self.AWGslider, 0, 1, 1, 2)
        self.AWGtext = QtWidgets.QDoubleSpinBox(self.frame_3)
        self.AWGtext.setMaximumSize(QtCore.QSize(150, 16777215))
        self.AWGtext.setMinimum(1.0)
        self.AWGtext.setMaximum(5000.0)
        self.AWGtext.setObjectName("AWGtext")
        self.gridLayout_3.addWidget(self.AWGtext, 0, 3, 1, 2)
        self.PV1text = QtWidgets.QDoubleSpinBox(self.frame_3)
        self.PV1text.setMaximumSize(QtCore.QSize(150, 16777215))
        self.PV1text.setMinimum(-5.0)
        self.PV1text.setMaximum(5.0)
        self.PV1text.setObjectName("PV1text")
        self.gridLayout_3.addWidget(self.PV1text, 2, 3, 1, 2)
        self.SQ1Label = QtWidgets.QPushButton(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.SQ1Label.sizePolicy().hasHeightForWidth())
        self.SQ1Label.setSizePolicy(sizePolicy)
        self.SQ1Label.setMaximumSize(QtCore.QSize(30, 16777215))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.SQ1Label.setFont(font)
        self.SQ1Label.setObjectName("SQ1Label")
        self.gridLayout_3.addWidget(self.SQ1Label, 1, 0, 1, 1)
        self.PV2Label = QtWidgets.QPushButton(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.PV2Label.sizePolicy().hasHeightForWidth())
        self.PV2Label.setSizePolicy(sizePolicy)
        self.PV2Label.setMaximumSize(QtCore.QSize(30, 16777215))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.PV2Label.setFont(font)
        self.PV2Label.setObjectName("PV2Label")
        self.gridLayout_3.addWidget(self.PV2Label, 3, 0, 1, 1)
        self.WGLabel = QtWidgets.QPushButton(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.WGLabel.sizePolicy().hasHeightForWidth())
        self.WGLabel.setSizePolicy(sizePolicy)
        self.WGLabel.setMaximumSize(QtCore.QSize(30, 16777215))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.WGLabel.setFont(font)
        self.WGLabel.setObjectName("WGLabel")
        self.gridLayout_3.addWidget(self.WGLabel, 0, 0, 1, 1)
        self.PV1Label = QtWidgets.QPushButton(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.PV1Label.sizePolicy().hasHeightForWidth())
        self.PV1Label.setSizePolicy(sizePolicy)
        self.PV1Label.setMaximumSize(QtCore.QSize(30, 16777215))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.PV1Label.setFont(font)
        self.PV1Label.setObjectName("PV1Label")
        self.gridLayout_3.addWidget(self.PV1Label, 2, 0, 1, 1)
        self.gridLayout_2.addWidget(self.frame_3, 10, 1, 2, 4)
        self.frame_6 = QtWidgets.QFrame(self.frame)
        self.frame_6.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_6.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_6.setObjectName("frame_6")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.frame_6)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.label_5 = QtWidgets.QLabel(self.frame_6)
        self.label_5.setObjectName("label_5")
        self.gridLayout_5.addWidget(self.label_5, 0, 0, 1, 1)
        self.horizontalSlider_6 = QtWidgets.QSlider(self.frame_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.horizontalSlider_6.sizePolicy().hasHeightForWidth())
        self.horizontalSlider_6.setSizePolicy(sizePolicy)
        self.horizontalSlider_6.setMinimum(-3300)
        self.horizontalSlider_6.setMaximum(3300)
        self.horizontalSlider_6.setProperty("value", 99)
        self.horizontalSlider_6.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSlider_6.setObjectName("horizontalSlider_6")
        self.gridLayout_5.addWidget(self.horizontalSlider_6, 1, 3, 1, 1)
        self.trigBox = QtWidgets.QComboBox(self.frame_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.trigBox.sizePolicy().hasHeightForWidth())
        self.trigBox.setSizePolicy(sizePolicy)
        self.trigBox.setMaximumSize(QtCore.QSize(55, 16777215))
        self.trigBox.setObjectName("trigBox")
        self.gridLayout_5.addWidget(self.trigBox, 1, 2, 1, 1)
        self.TBslider = QtWidgets.QSlider(self.frame_6)
        self.TBslider.setMaximum(8)
        self.TBslider.setOrientation(QtCore.Qt.Horizontal)
        self.TBslider.setTickPosition(QtWidgets.QSlider.TicksAbove)
        self.TBslider.setObjectName("TBslider")
        self.gridLayout_5.addWidget(self.TBslider, 0, 1, 1, 3)
        self.trigEnable = QtWidgets.QCheckBox(self.frame_6)
        self.trigEnable.setChecked(True)
        self.trigEnable.setObjectName("trigEnable")
        self.gridLayout_5.addWidget(self.trigEnable, 1, 0, 1, 1)
        self.gridLayout_2.addWidget(self.frame_6, 15, 1, 1, 4)
        self.pushButton_4 = QtWidgets.QPushButton(self.frame)
        self.pushButton_4.setObjectName("pushButton_4")
        self.gridLayout_2.addWidget(self.pushButton_4, 16, 1, 1, 3)
        self.CCS = QtWidgets.QCheckBox(self.frame)
        self.CCS.setObjectName("CCS")
        self.gridLayout_2.addWidget(self.CCS, 7, 3, 1, 2)
        self.line_2 = QtWidgets.QFrame(self.frame)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.gridLayout_2.addWidget(self.line_2, 8, 1, 1, 4)
        self.pushButton_5 = QtWidgets.QPushButton(self.frame)
        self.pushButton_5.setObjectName("pushButton_5")
        self.gridLayout_2.addWidget(self.pushButton_5, 16, 4, 1, 1)
        self.line_3 = QtWidgets.QFrame(self.frame)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.gridLayout_2.addWidget(self.line_3, 12, 1, 1, 4)
        self.RES = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.RES.sizePolicy().hasHeightForWidth())
        self.RES.setSizePolicy(sizePolicy)
        self.RES.setObjectName("RES")
        self.gridLayout_2.addWidget(self.RES, 3, 1, 1, 4)
        self.FREQ = QtWidgets.QPushButton(self.frame)
        self.FREQ.setObjectName("FREQ")
        self.gridLayout_2.addWidget(self.FREQ, 5, 1, 1, 4)
        self.OD1 = QtWidgets.QCheckBox(self.frame)
        self.OD1.setObjectName("OD1")
        self.gridLayout_2.addWidget(self.OD1, 7, 1, 1, 2)
        self.line = QtWidgets.QFrame(self.frame)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_2.addWidget(self.line, 6, 1, 1, 4)
        self.comboBox_2 = QtWidgets.QComboBox(self.frame)
        self.comboBox_2.setObjectName("comboBox_2")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.gridLayout_2.addWidget(self.comboBox_2, 9, 4, 1, 1)
        self.Wshape = QtWidgets.QComboBox(self.frame)
        self.Wshape.setObjectName("Wshape")
        self.Wshape.addItem("")
        self.Wshape.addItem("")
        self.Wshape.addItem("")
        self.gridLayout_2.addWidget(self.Wshape, 9, 1, 1, 3)
        self.CAP = QtWidgets.QPushButton(self.frame)
        self.CAP.setObjectName("CAP")
        self.gridLayout_2.addWidget(self.CAP, 4, 1, 1, 4)
        self.pushButton = QtWidgets.QPushButton(self.frame)
        self.pushButton.setObjectName("pushButton")
        self.gridLayout_2.addWidget(self.pushButton, 0, 4, 1, 1)
        self.frame_7 = QtWidgets.QFrame(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_7.sizePolicy().hasHeightForWidth())
        self.frame_7.setSizePolicy(sizePolicy)
        self.frame_7.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_7.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_7.setObjectName("frame_7")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.frame_7)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(3)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.voltMeterCB1 = QtWidgets.QCheckBox(self.frame_7)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.voltMeterCB1.setFont(font)
        self.voltMeterCB1.setObjectName("voltMeterCB1")
        self.horizontalLayout.addWidget(self.voltMeterCB1)
        self.voltMeterCB2 = QtWidgets.QCheckBox(self.frame_7)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.voltMeterCB2.setFont(font)
        self.voltMeterCB2.setObjectName("voltMeterCB2")
        self.horizontalLayout.addWidget(self.voltMeterCB2)
        self.voltMeterCB3 = QtWidgets.QCheckBox(self.frame_7)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.voltMeterCB3.setFont(font)
        self.voltMeterCB3.setObjectName("voltMeterCB3")
        self.horizontalLayout.addWidget(self.voltMeterCB3)
        self.gridLayout_2.addWidget(self.frame_7, 2, 1, 1, 4)
        self.gridLayout.addWidget(self.splitter, 0, 1, 1, 4)
        self.frame_2 = QtWidgets.QFrame(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
        self.frame_2.setSizePolicy(sizePolicy)
        self.frame_2.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.offsetLayout = QtWidgets.QVBoxLayout(self.frame_2)
        self.offsetLayout.setContentsMargins(0, 0, 0, 0)
        self.offsetLayout.setSpacing(3)
        self.offsetLayout.setObjectName("offsetLayout")
        self.slider1 = QtWidgets.QSlider(self.frame_2)
        self.slider1.setStyleSheet("")
        self.slider1.setMinimum(-4)
        self.slider1.setMaximum(4)
        self.slider1.setOrientation(QtCore.Qt.Vertical)
        self.slider1.setObjectName("slider1")
        self.offsetLayout.addWidget(self.slider1)
        self.slider2 = QtWidgets.QSlider(self.frame_2)
        self.slider2.setStyleSheet("")
        self.slider2.setMinimum(-4)
        self.slider2.setMaximum(4)
        self.slider2.setOrientation(QtCore.Qt.Vertical)
        self.slider2.setObjectName("slider2")
        self.offsetLayout.addWidget(self.slider2)
        self.slider3 = QtWidgets.QSlider(self.frame_2)
        self.slider3.setStyleSheet("")
        self.slider3.setMinimum(-4)
        self.slider3.setMaximum(4)
        self.slider3.setOrientation(QtCore.Qt.Vertical)
        self.slider3.setObjectName("slider3")
        self.offsetLayout.addWidget(self.slider3)
        self.slider4 = QtWidgets.QSlider(self.frame_2)
        self.slider4.setStyleSheet("")
        self.slider4.setMinimum(-4)
        self.slider4.setMaximum(4)
        self.slider4.setOrientation(QtCore.Qt.Vertical)
        self.slider4.setObjectName("slider4")
        self.offsetLayout.addWidget(self.slider4)
        self.gridLayout.addWidget(self.frame_2, 0, 0, 1, 1)
        self.Freeze = QtWidgets.QCheckBox(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Freeze.sizePolicy().hasHeightForWidth())
        self.Freeze.setSizePolicy(sizePolicy)
        self.Freeze.setObjectName("Freeze")
        self.gridLayout.addWidget(self.Freeze, 1, 4, 1, 1)
        self.Cross = QtWidgets.QCheckBox(Form)
        self.Cross.setMaximumSize(QtCore.QSize(70, 16777215))
        self.Cross.setObjectName("Cross")
        self.gridLayout.addWidget(self.Cross, 1, 2, 1, 1)

        self.retranslateUi(Form)
        self.A2Range.setCurrentIndex(2)
        self.A1Map.setCurrentIndex(-1)
        self.A1Range.setCurrentIndex(2)
        self.A3Range.setCurrentIndex(0)
        self.MICRange.setCurrentIndex(0)
        self.comboBox_2.setCurrentIndex(2)
        self.pushButton_5.clicked.connect(Form.show_fft)
        self.pushButton_4.clicked.connect(Form.save_data)
        self.Wshape.currentIndexChanged['int'].connect(Form.select_wave)
        self.comboBox_2.currentIndexChanged['int'].connect(Form.select_wgain)
        self.AWGslider.valueChanged['int'].connect(Form.awg_slider)
        self.AWGtext.editingFinished.connect(Form.awg_text)
        self.SQ1DCtext.editingFinished.connect(Form.sq1_dc)
        self.SQ1text.editingFinished.connect(Form.sq1_text)
        self.SQ1slider.valueChanged['int'].connect(Form.sq1_slider)
        self.PV1text.editingFinished.connect(Form.pv1_text)
        self.PV2text.editingFinished.connect(Form.pv2_text)
        self.PV1slider.valueChanged['int'].connect(Form.pv1_slider)
        self.PV2slider.valueChanged['int'].connect(Form.pv2_slider)
        self.TBslider.valueChanged['int'].connect(Form.set_timebase)
        self.trigBox.currentIndexChanged['int'].connect(Form.select_trig_source)
        self.horizontalSlider_6.valueChanged['int'].connect(Form.set_trigger)
        self.pushButton.clicked.connect(Form.show_voltmeter)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.Diff.setText(_translate("Form", "A2-A1"))
        self.msgwin.setText(_translate("Form", "."))
        self.label_4.setText(_translate("Form", "Measure Voltages"))
        self.A2Range.setItemText(0, _translate("Form", "16V"))
        self.A2Range.setItemText(1, _translate("Form", "8V"))
        self.A2Range.setItemText(2, _translate("Form", "4V"))
        self.A2Range.setItemText(3, _translate("Form", "2.5V"))
        self.A2Range.setItemText(4, _translate("Form", "1V"))
        self.A2Range.setItemText(5, _translate("Form", "0.5V"))
        self.A3Box.setText(_translate("Form", "A3"))
        self.A1Range.setItemText(0, _translate("Form", "16V"))
        self.A1Range.setItemText(1, _translate("Form", "8V"))
        self.A1Range.setItemText(2, _translate("Form", "4V"))
        self.A1Range.setItemText(3, _translate("Form", "2.5V"))
        self.A1Range.setItemText(4, _translate("Form", "1V"))
        self.A1Range.setItemText(5, _translate("Form", "0.5V"))
        self.A2Box.setText(_translate("Form", "A2"))
        self.A3Range.setItemText(0, _translate("Form", "4V"))
        self.A3Range.setItemText(1, _translate("Form", "2V"))
        self.A3Range.setItemText(2, _translate("Form", "1V"))
        self.A3Range.setItemText(3, _translate("Form", "0.5V"))
        self.MICRange.setItemText(0, _translate("Form", "4V"))
        self.MICRange.setItemText(1, _translate("Form", "2V"))
        self.MICRange.setItemText(2, _translate("Form", "1V"))
        self.MICRange.setItemText(3, _translate("Form", "0.5V"))
        self.MICBox.setText(_translate("Form", "MIC"))
        self.PV2text.setSuffix(_translate("Form", " Volts"))
        self.SQ1DCtext.setSuffix(_translate("Form", " %"))
        self.SQ1text.setSuffix(_translate("Form", " Hz"))
        self.AWGtext.setSuffix(_translate("Form", " Hz"))
        self.PV1text.setSuffix(_translate("Form", " Volts"))
        self.SQ1Label.setText(_translate("Form", "SQ1"))
        self.PV2Label.setText(_translate("Form", "PV2"))
        self.WGLabel.setText(_translate("Form", "WG"))
        self.PV1Label.setText(_translate("Form", "PV1"))
        self.label_5.setText(_translate("Form", "Timebase"))
        self.trigEnable.setText(_translate("Form", "Trigger"))
        self.pushButton_4.setText(_translate("Form", "SAVE Traces"))
        self.CCS.setText(_translate("Form", "CCS"))
        self.pushButton_5.setText(_translate("Form", "Fourier Transform"))
        self.RES.setText(_translate("Form", "Resistance on SEN"))
        self.FREQ.setText(_translate("Form", "Frequency (IN2):"))
        self.OD1.setText(_translate("Form", "OD1"))
        self.comboBox_2.setItemText(0, _translate("Form", "80 mV"))
        self.comboBox_2.setItemText(1, _translate("Form", "1V"))
        self.comboBox_2.setItemText(2, _translate("Form", "3V (Amplitude)"))
        self.Wshape.setItemText(0, _translate("Form", "WG( Sinusoidal )"))
        self.Wshape.setItemText(1, _translate("Form", "WG( Triangle )"))
        self.Wshape.setItemText(2, _translate("Form", "SQ2( Square )"))
        self.CAP.setText(_translate("Form", "Capacitance (IN1):"))
        self.pushButton.setText(_translate("Form", "SHOW ALL"))
        self.voltMeterCB1.setText(_translate("Form", "A1"))
        self.voltMeterCB2.setText(_translate("Form", "A2"))
        self.voltMeterCB3.setText(_translate("Form", "A3"))
        self.Freeze.setText(_translate("Form", "FREEZE"))
        self.Cross.setText(_translate("Form", "Cursor"))

from pyqtgraph import PlotWidget



#from __future__ import print_function
import os, sys, time, utils, inspect, os.path

from QtVersion import *

class ListStream:
    def __init__(self):
            self.data = ''
    def write(self, s):
            self.data += s


class Expt(QWidget):
	functionList = {}
	mycode = ''
	TIMER = 500
	ht = 40
	info = 'Gravity by Time of Flight.\nConnect the electromagnet to OD1 and Attach the ball.\nPlace the contact sensor below and connect it between SEN and GND.\nPress the Measure button.'
	
	def __init__(self, device=None):
		QWidget.__init__(self)
		self.p = device										# connection to the device hardware 		


		full = QVBoxLayout()
		
		self.Edit = QTextEdit()	
		full.addWidget(self.Edit)
		font = QFont()
		font.setPointSize(16)
		self.Edit.setFont(font)

		s=self.tr(self.info)
		self.Edit.setText(s)
		self.p.set_state(OD1=1)
			
		H = QHBoxLayout()
		b = QPushButton(self.tr("Measure"))
		H.addWidget(b)
		b.clicked.connect(self.measure_tof)		
		H.addWidget(b)
		full.addLayout(H)


		self.msgwin = QLabel(text='')
		self.msgwin.setStyleSheet('background-color: white')
		full.addWidget(self.msgwin)
				
		self.setLayout(full)	
		self.timer = QTimer()

		#----------------------------- end of init ---------------

	def ht_text(self, text):
		try:
			val = float(text)
		except:
			return
		val = float(text)
		print (val)
			
		
	def measure_tof(self):
		self.msg(self.tr('start..'))
		res = self.p.SinglePinEdges('SEN','falling',1,OD1=0)
		self.p.set_state(OD1=1)
		try:
			t = res[0]
			ss = '%5.3f'%t
			self.msg(self.tr('Time of flight =') + ss + self.tr(' Seconds'))
		except:
			self.msg(self.tr('Error. Try again'))
			
	def sq1_slider(self, val):
		try:
			res = self.p.set_sqr1(val*0.01)
			ss = '%5.2f'%res
			self.msg(self.tr('sqr1 set to ') + ss + self.tr(' Hz'))
		except:
			self.comerr()
					
	def msg(self, m):
		self.msgwin.setText(self.tr(m))
		
	def comerr(self):
		self.msgwin.setText('<font color="red">' + self.tr('Error. Try Device->Reconnect'))

if __name__ == '__main__':
	import eyes17.eyes
	dev = eyes17.eyes.open()
	app = QApplication(sys.argv)

	# translation stuff
	lang=QLocale.system().name()
	t=QTranslator()
	t.load("lang/"+lang, os.path.dirname(__file__))
	app.installTranslator(t)
	t1=QTranslator()
	t1.load("qt_"+lang,
	        QLibraryInfo.location(QLibraryInfo.TranslationsPath))
	app.installTranslator(t1)

	mw = Expt(dev)
	mw.show()
	sys.exit(app.exec_())
	



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dio_sensor.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

import sys
if sys.version_info.major==3:
	from PyQt5 import QtGui, QtCore, QtWidgets
else:
	from PyQt4 import QtGui, QtCore
	from PyQt4 import QtGui as QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(900, 600)
        font = QtGui.QFont()
        font.setPointSize(9)
        Dialog.setFont(font)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/control/play.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Dialog.setWindowIcon(icon)
        Dialog.setAutoFillBackground(True)
        Dialog.setSizeGripEnabled(True)
        Dialog.setModal(False)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setContentsMargins(3, 3, 3, 0)
        self.gridLayout.setSpacing(3)
        self.gridLayout.setObjectName("gridLayout")
        self.configLayout = QtWidgets.QHBoxLayout()
        self.configLayout.setObjectName("configLayout")
        self.gridLayout.addLayout(self.configLayout, 1, 0, 1, 4)
        self.switcher = QtWidgets.QPushButton(Dialog)
        self.switcher.setObjectName("switcher")
        self.gridLayout.addWidget(self.switcher, 0, 3, 1, 1)
        self.pushButton = QtWidgets.QPushButton(Dialog)
        self.pushButton.setObjectName("pushButton")
        self.gridLayout.addWidget(self.pushButton, 0, 0, 1, 1)
        self.log_2 = QtWidgets.QCheckBox(Dialog)
        self.log_2.setObjectName("log_2")
        self.gridLayout.addWidget(self.log_2, 0, 1, 1, 1)
        self.monitors = QtWidgets.QStackedWidget(Dialog)
        self.monitors.setObjectName("monitors")
        self.page = QtWidgets.QWidget()
        self.page.setObjectName("page")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.page)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gaugeLayout = QtWidgets.QGridLayout()
        self.gaugeLayout.setSpacing(3)
        self.gaugeLayout.setObjectName("gaugeLayout")
        self.verticalLayout.addLayout(self.gaugeLayout)
        self.monitors.addWidget(self.page)
        self.page_2 = QtWidgets.QWidget()
        self.page_2.setObjectName("page_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.page_2)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setHorizontalSpacing(2)
        self.gridLayout_2.setVerticalSpacing(0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.pushButton_2 = QtWidgets.QPushButton(self.page_2)
        self.pushButton_2.setObjectName("pushButton_2")
        self.gridLayout_2.addWidget(self.pushButton_2, 2, 1, 1, 1)
        self.pauseBox = QtWidgets.QCheckBox(self.page_2)
        self.pauseBox.setMaximumSize(QtCore.QSize(70, 16777215))
        self.pauseBox.setObjectName("pauseBox")
        self.gridLayout_2.addWidget(self.pauseBox, 2, 0, 1, 1)
        self.pushButton_3 = QtWidgets.QPushButton(self.page_2)
        self.pushButton_3.setObjectName("pushButton_3")
        self.gridLayout_2.addWidget(self.pushButton_3, 2, 2, 1, 1)
        self.graph = PlotWidget(self.page_2)
        self.graph.setObjectName("graph")
        self.gridLayout_2.addWidget(self.graph, 1, 0, 1, 3)
        self.monitors.addWidget(self.page_2)
        self.gridLayout.addWidget(self.monitors, 2, 0, 1, 5)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 2, 1, 1)

        self.retranslateUi(Dialog)
        self.monitors.setCurrentIndex(0)
        self.log_2.toggled['bool'].connect(Dialog.changeRange)
        self.pushButton.clicked.connect(Dialog.initialize)
        self.switcher.clicked.connect(Dialog.next)
        self.pushButton_2.clicked.connect(Dialog.sineFit)
        self.pauseBox.toggled['bool'].connect(Dialog.pause)
        self.pushButton_3.clicked.connect(Dialog.dampedSineFit)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Sensor Reading"))
        self.switcher.setText(_translate("Dialog", "Data Logger"))
        self.pushButton.setText(_translate("Dialog", "Initialize"))
        self.log_2.setText(_translate("Dialog", "Scale"))
        self.pushButton_2.setText(_translate("Dialog", "SINE FIT"))
        self.pauseBox.setText(_translate("Dialog", "Pause"))
        self.pushButton_3.setText(_translate("Dialog", "DAMPED SINE FIT"))

from pyqtgraph import PlotWidget
from . import res_rc
